@@ -1,5 +1,5 @@
 #! /bin/sh
 
-cpp -P ${srcdir:-.}/gdk-pixbuf.symbols | sed -e '/^$/d' -e 's/ G_GNUC.*$//' | sort > expected-abi
+cpp -P ${srcdir:-.}/gdk-pixbuf.symbols | sed -e '/^$/d' -e 's/ G_GNUC.*$//' -e 's/ PRIVATE$//' | sort > expected-abi
 nm -D .libs/libgdk_pixbuf-2.0.so | grep " T " | cut -d ' ' -f 3 | sort > actual-abi
 diff -u expected-abi actual-abi && rm expected-abi actual-abi
