@@ -37,6 +37,8 @@
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-io.h"
 
+#include <glib/gstdio.h>
+
 #ifdef G_OS_WIN32
 #define STRICT
 #include <windows.h>
@@ -79,12 +81,17 @@ format_check (GdkPixbufModule *module, guchar *buffer, int size)
 G_LOCK_DEFINE_STATIC (init_lock);
 G_LOCK_DEFINE_STATIC (threadunsafe_loader_lock);
 
-void
+gboolean
 _gdk_pixbuf_lock (GdkPixbufModule *image_module)
 {
- 	if (!(image_module->info->flags & GDK_PIXBUF_FORMAT_THREADSAFE)) {
+ 	if (g_threads_got_initialized &&
+	    !(image_module->info->flags & GDK_PIXBUF_FORMAT_THREADSAFE)) {
  		G_LOCK (threadunsafe_loader_lock);
+
+		return TRUE;
  	}
+
+	return FALSE;
 }
  
 void
@@ -243,7 +250,7 @@ correct_prefix (gchar **path)
     }
 }
 
-#endif
+#endif  /* G_OS_WIN32 */
 
 static gchar *
 gdk_pixbuf_get_module_file (void)
@@ -424,7 +431,7 @@ _gdk_pixbuf_load_module_unlocked (GdkPixbufModule *image_module,
 	char *path;
 	GModule *module;
 	gpointer sym;
-	
+		
         g_return_val_if_fail (image_module->module == NULL, FALSE);
 
 	path = image_module->module_path;
@@ -460,14 +467,23 @@ _gdk_pixbuf_load_module (GdkPixbufModule *image_module,
 			 GError         **error)
 {
 	gboolean ret;
+	gboolean locked = FALSE;
 
-	G_LOCK (init_lock);
+	/* be extra careful, maybe the module initializes
+	 * the thread system
+	 */
+	if (g_threads_got_initialized)
+	{
+		G_LOCK (init_lock);
+		locked = TRUE;
+	}
 	ret = _gdk_pixbuf_load_module_unlocked (image_module, error);
-	G_UNLOCK (init_lock);
-
+	if (locked)
+		G_UNLOCK (init_lock);
 	return ret;
 }
-#else
+
+#else  /* !USE_GMODULE */
 
 #define module(type) \
   extern void MODULE_ENTRY (type, fill_info)   (GdkPixbufFormat *info);   \
@@ -631,14 +647,14 @@ gdk_pixbuf_io_init ()
 	for (name = included_formats; *name; name++) {
 		module = g_new0 (GdkPixbufModule, 1);
 		module->module_name = *name;
-		if (_gdk_pixbuf_load_module_unlocked (module, NULL))
+		if (_gdk_pixbuf_load_module (module, NULL))
 			file_formats = g_slist_prepend (file_formats, module);
 		else
 			g_free (module);
 	}
 }
 
-#endif
+#endif  /* !USE_GMODULE */
 
 
 
@@ -736,8 +752,9 @@ _gdk_pixbuf_generic_image_load (GdkPixbufModule *module,
 	GdkPixbuf *pixbuf = NULL;
 	GdkPixbufAnimation *animation = NULL;
 	gpointer context;
+	gboolean locked;
 
-	_gdk_pixbuf_lock (module);
+	locked = _gdk_pixbuf_lock (module);
 
 	if (module->load != NULL) {
 		pixbuf = (* module->load) (f, error);
@@ -773,19 +790,19 @@ _gdk_pixbuf_generic_image_load (GdkPixbufModule *module,
 			pixbuf = gdk_pixbuf_animation_get_static_image (animation);
 
 			g_object_ref (pixbuf);
-
 			g_object_unref (animation);
 		}
 	}
 
  out:
-	_gdk_pixbuf_unlock (module);
+	if (locked)
+		_gdk_pixbuf_unlock (module);
 	return pixbuf;
 }
 
 /**
  * gdk_pixbuf_new_from_file:
- * @filename: Name of file to load.
+ * @filename: Name of file to load, in the GLib file name encoding
  * @error: Return location for an error
  *
  * Creates a new pixbuf by loading an image from a file.  The file format is
@@ -813,7 +830,7 @@ gdk_pixbuf_new_from_file (const char *filename,
 	
 	display_name = g_filename_display_name (filename);	
 
-	f = fopen (filename, "rb");
+	f = g_fopen (filename, "rb");
 	if (!f) {
                 g_set_error (error,
                              G_FILE_ERROR,
@@ -886,6 +903,29 @@ gdk_pixbuf_new_from_file (const char *filename,
 	return pixbuf;
 }
 
+#ifdef G_OS_WIN32
+
+#undef gdk_pixbuf_new_from_file
+
+GdkPixbuf *
+gdk_pixbuf_new_from_file (const char *filename,
+                          GError    **error)
+{
+	gchar *utf8_filename =
+		g_locale_to_utf8 (filename, -1, NULL, NULL, error);
+	GdkPixbuf *retval;
+
+	if (utf8_filename == NULL)
+		return NULL;
+
+	retval = gdk_pixbuf_new_from_file_utf8 (utf8_filename, error);
+
+	g_free (utf8_filename);
+
+	return retval;
+}
+#endif
+
 static void
 size_prepared_cb (GdkPixbufLoader *loader, 
 		  int              width,
@@ -917,9 +957,68 @@ size_prepared_cb (GdkPixbufLoader *loader,
 	gdk_pixbuf_loader_set_size (loader, width, height);
 }
 
+/**
+ * gdk_pixbuf_new_from_file_at_size:
+ * @filename: Name of file to load, in the GLib file name encoding
+ * @width: The width the image should have
+ * @height: The height the image should have
+ * @error: Return location for an error
+ *
+ * Creates a new pixbuf by loading an image from a file.  The file format is
+ * detected automatically. If %NULL is returned, then @error will be set.
+ * Possible errors are in the #GDK_PIXBUF_ERROR and #G_FILE_ERROR domains.
+ * The image will be scaled to fit in the requested size, preserving
+ * the image's aspect ratio.
+ *
+ * Return value: A newly-created pixbuf with a reference count of 1, or 
+ * %NULL if any of several error conditions occurred:  the file could not 
+ * be opened, there was no loader for the file's format, there was not 
+ * enough memory to allocate the image buffer, or the image file contained 
+ * invalid data.
+ *
+ * Since: 2.4
+ **/
+GdkPixbuf *
+gdk_pixbuf_new_from_file_at_size (const char *filename,
+				  int         width, 
+				  int         height,
+				  GError    **error)
+{
+	return gdk_pixbuf_new_from_file_at_scale (filename, 
+						  width, height, 
+						  TRUE, error);
+}
+
+#ifdef G_OS_WIN32
+
+#undef gdk_pixbuf_new_from_file_at_size
+
+GdkPixbuf *
+gdk_pixbuf_new_from_file_at_size (const char *filename,
+				  int         width, 
+				  int         height,
+				  GError    **error)
+{
+	gchar *utf8_filename =
+		g_locale_to_utf8 (filename, -1, NULL, NULL, error);
+	GdkPixbuf *retval;
+
+	if (utf8_filename == NULL)
+		return NULL;
+
+	retval = gdk_pixbuf_new_from_file_at_size_utf8 (utf8_filename,
+							width, height,
+							error);
+
+	g_free (utf8_filename);
+
+	return retval;
+}
+#endif
+
 /**
  * gdk_pixbuf_new_from_file_at_scale:
- * @filename: Name of file to load.
+ * @filename: Name of file to load, in the GLib file name encoding
  * @width: The width the image should have
  * @height: The height the image should have
  * @preserve_aspect_ratio: %TRUE to preserve the image's aspect ratio
@@ -961,7 +1060,7 @@ gdk_pixbuf_new_from_file_at_scale (const char *filename,
 	g_return_val_if_fail (filename != NULL, NULL);
         g_return_val_if_fail (width > 0 && height > 0, NULL);
 
-	f = fopen (filename, "rb");
+	f = g_fopen (filename, "rb");
 	if (!f) {
                 gchar *display_name = g_filename_display_name (filename);
                 g_set_error (error,
@@ -1021,37 +1120,35 @@ gdk_pixbuf_new_from_file_at_scale (const char *filename,
 	return pixbuf;
 }
 
-/**
- * gdk_pixbuf_new_from_file_at_size:
- * @filename: Name of file to load.
- * @width: The width the image should have
- * @height: The height the image should have
- * @error: Return location for an error
- *
- * Creates a new pixbuf by loading an image from a file.  The file format is
- * detected automatically. If %NULL is returned, then @error will be set.
- * Possible errors are in the #GDK_PIXBUF_ERROR and #G_FILE_ERROR domains.
- * The image will be scaled to fit in the requested size, preserving
- * the image's aspect ratio.
- *
- * Return value: A newly-created pixbuf with a reference count of 1, or 
- * %NULL if any of several error conditions occurred:  the file could not 
- * be opened, there was no loader for the file's format, there was not 
- * enough memory to allocate the image buffer, or the image file contained 
- * invalid data.
- *
- * Since: 2.4
- **/
+#ifdef G_OS_WIN32
+
+#undef gdk_pixbuf_new_from_file_at_scale
+
 GdkPixbuf *
-gdk_pixbuf_new_from_file_at_size (const char *filename,
-				  int         width, 
-				  int         height,
-				  GError    **error)
+gdk_pixbuf_new_from_file_at_scale (const char *filename,
+				   int         width, 
+				   int         height,
+				   gboolean    preserve_aspect_ratio,
+				   GError    **error)
 {
-	return gdk_pixbuf_new_from_file_at_scale (filename, 
-						  width, height, 
-						  TRUE, error);
+	gchar *utf8_filename =
+		g_locale_to_utf8 (filename, -1, NULL, NULL, error);
+	GdkPixbuf *retval;
+
+	if (utf8_filename == NULL)
+		return NULL;
+
+	retval = gdk_pixbuf_new_from_file_at_scale_utf8 (utf8_filename,
+							 width, height,
+							 preserve_aspect_ratio,
+							 error);
+
+	g_free (utf8_filename);
+
+	return retval;
 }
+#endif
+
 
 static void
 info_cb (GdkPixbufLoader *loader, 
@@ -1105,7 +1202,7 @@ gdk_pixbuf_get_file_info (const gchar  *filename,
 
 	g_return_val_if_fail (filename != NULL, NULL);
 
-	f = fopen (filename, "rb");
+	f = g_fopen (filename, "rb");
 	if (!f)
 		return NULL;
 
@@ -1154,7 +1251,10 @@ gdk_pixbuf_new_from_xpm_data (const char **data)
 	GdkPixbuf *(* load_xpm_data) (const char **data);
 	GdkPixbuf *pixbuf;
         GError *error = NULL;
-	GdkPixbufModule *xpm_module = _gdk_pixbuf_get_named_module ("xpm", &error);
+	GdkPixbufModule *xpm_module;
+	gboolean locked;
+
+	xpm_module = _gdk_pixbuf_get_named_module ("xpm", &error);
 	if (xpm_module == NULL) {
 		g_warning ("Error loading XPM image loader: %s", error->message);
 		g_error_free (error);
@@ -1169,7 +1269,7 @@ gdk_pixbuf_new_from_xpm_data (const char **data)
                 }
         }
 
-	_gdk_pixbuf_lock (xpm_module);
+	locked = _gdk_pixbuf_lock (xpm_module);
 
 	if (xpm_module->load_xpm_data == NULL) {
 		g_warning ("gdk-pixbuf XPM module lacks XPM data capability");
@@ -1178,8 +1278,9 @@ gdk_pixbuf_new_from_xpm_data (const char **data)
 		load_xpm_data = xpm_module->load_xpm_data;
 		pixbuf = (* load_xpm_data) (data);
 	}
-
-	_gdk_pixbuf_unlock (xpm_module);
+	
+	if (locked)
+		_gdk_pixbuf_unlock (xpm_module);
 	return pixbuf;
 }
 
@@ -1250,6 +1351,7 @@ gdk_pixbuf_real_save (GdkPixbuf     *pixbuf,
 {
 	gboolean ret;
 	GdkPixbufModule *image_module = NULL;       
+	gboolean locked;
 
 	image_module = _gdk_pixbuf_get_named_module (type, error);
 
@@ -1260,7 +1362,7 @@ gdk_pixbuf_real_save (GdkPixbuf     *pixbuf,
 		if (!_gdk_pixbuf_load_module (image_module, error))
 			return FALSE;
 
-	_gdk_pixbuf_lock (image_module);
+	locked = _gdk_pixbuf_lock (image_module);
 
 	if (image_module->save) {
 		/* save normally */
@@ -1283,7 +1385,8 @@ gdk_pixbuf_real_save (GdkPixbuf     *pixbuf,
 		ret = FALSE;
 	}
 
-	_gdk_pixbuf_unlock (image_module);
+	if (locked)
+		_gdk_pixbuf_unlock (image_module);
 	return ret;
 }
 
@@ -1304,6 +1407,7 @@ save_to_callback_with_tmp_file (GdkPixbufModule   *image_module,
 	gchar *buf = NULL;
 	gsize n;
 	gchar *filename = NULL;
+	gboolean locked;
 
 	buf = g_try_malloc (TMP_FILE_BUF_SIZE);
 	if (buf == NULL) {
@@ -1326,9 +1430,10 @@ save_to_callback_with_tmp_file (GdkPixbufModule   *image_module,
 		goto end;
 	}
 
-	_gdk_pixbuf_lock (image_module);
+	locked = _gdk_pixbuf_lock (image_module);
 	retval = (image_module->save) (f, pixbuf, keys, values, error);
-	_gdk_pixbuf_unlock (image_module);
+	if (locked)
+		_gdk_pixbuf_unlock (image_module);
 	if (!retval)
 		goto end;
 
@@ -1356,7 +1461,7 @@ save_to_callback_with_tmp_file (GdkPixbufModule   *image_module,
 	if (f)
 		fclose (f);
 	if (filename) {
-		unlink (filename);
+		g_unlink (filename);
 		g_free (filename);
 	}
 	g_free (buf);
@@ -1375,6 +1480,7 @@ gdk_pixbuf_real_save_to_callback (GdkPixbuf         *pixbuf,
 {
 	gboolean ret;
 	GdkPixbufModule *image_module = NULL;       
+	gboolean locked;
 
 	image_module = _gdk_pixbuf_get_named_module (type, error);
 
@@ -1385,7 +1491,7 @@ gdk_pixbuf_real_save_to_callback (GdkPixbuf         *pixbuf,
 		if (!_gdk_pixbuf_load_module (image_module, error))
 			return FALSE;
 
-	_gdk_pixbuf_lock (image_module);
+	locked = _gdk_pixbuf_lock (image_module);
 
 	if (image_module->save_to_callback) {
 		/* save normally */
@@ -1408,7 +1514,8 @@ gdk_pixbuf_real_save_to_callback (GdkPixbuf         *pixbuf,
 		ret = FALSE;
 	}
 
-	_gdk_pixbuf_unlock (image_module);
+	if (locked)
+		_gdk_pixbuf_unlock (image_module);
 	return ret;
 }
 
@@ -1524,7 +1631,7 @@ gdk_pixbuf_savev (GdkPixbuf  *pixbuf,
         g_return_val_if_fail (type != NULL, FALSE);
         g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
        
-        f = fopen (filename, "wb");
+        f = g_fopen (filename, "wb");
         
         if (f == NULL) {
                 gchar *display_name = g_filename_display_name (filename);
