@@ -2,6 +2,7 @@
 /* Generated data (by glib-mkenums) */
 
 #define GDK_ENABLE_BROKEN
+#include "gdkalias.h"
 #include "gdk.h"
 
 /* enumerations from "gdkcursor.h" */
@@ -197,6 +198,7 @@ gdk_event_type_get_type (void)
       { GDK_SCROLL, "GDK_SCROLL", "scroll" },
       { GDK_WINDOW_STATE, "GDK_WINDOW_STATE", "window-state" },
       { GDK_SETTING, "GDK_SETTING", "setting" },
+      { GDK_OWNER_CHANGE, "GDK_OWNER_CHANGE", "owner-change" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GdkEventType", values);
@@ -332,6 +334,9 @@ gdk_window_state_get_type (void)
       { GDK_WINDOW_STATE_ICONIFIED, "GDK_WINDOW_STATE_ICONIFIED", "iconified" },
       { GDK_WINDOW_STATE_MAXIMIZED, "GDK_WINDOW_STATE_MAXIMIZED", "maximized" },
       { GDK_WINDOW_STATE_STICKY, "GDK_WINDOW_STATE_STICKY", "sticky" },
+      { GDK_WINDOW_STATE_FULLSCREEN, "GDK_WINDOW_STATE_FULLSCREEN", "fullscreen" },
+      { GDK_WINDOW_STATE_ABOVE, "GDK_WINDOW_STATE_ABOVE", "above" },
+      { GDK_WINDOW_STATE_BELOW, "GDK_WINDOW_STATE_BELOW", "below" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GdkWindowState", values);
@@ -355,6 +360,22 @@ gdk_setting_action_get_type (void)
   return etype;
 }
 
+GType
+gdk_owner_change_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GDK_OWNER_CHANGE_NEW_OWNER, "GDK_OWNER_CHANGE_NEW_OWNER", "new-owner" },
+      { GDK_OWNER_CHANGE_DESTROY, "GDK_OWNER_CHANGE_DESTROY", "destroy" },
+      { GDK_OWNER_CHANGE_CLOSE, "GDK_OWNER_CHANGE_CLOSE", "close" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GdkOwnerChange", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "gdkfont.h" */
 GType
@@ -881,6 +902,10 @@ gdk_window_type_hint_get_type (void)
       { GDK_WINDOW_TYPE_HINT_DIALOG, "GDK_WINDOW_TYPE_HINT_DIALOG", "dialog" },
       { GDK_WINDOW_TYPE_HINT_MENU, "GDK_WINDOW_TYPE_HINT_MENU", "menu" },
       { GDK_WINDOW_TYPE_HINT_TOOLBAR, "GDK_WINDOW_TYPE_HINT_TOOLBAR", "toolbar" },
+      { GDK_WINDOW_TYPE_HINT_SPLASHSCREEN, "GDK_WINDOW_TYPE_HINT_SPLASHSCREEN", "splashscreen" },
+      { GDK_WINDOW_TYPE_HINT_UTILITY, "GDK_WINDOW_TYPE_HINT_UTILITY", "utility" },
+      { GDK_WINDOW_TYPE_HINT_DOCK, "GDK_WINDOW_TYPE_HINT_DOCK", "dock" },
+      { GDK_WINDOW_TYPE_HINT_DESKTOP, "GDK_WINDOW_TYPE_HINT_DESKTOP", "desktop" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GdkWindowTypeHint", values);
