@@ -26,6 +26,8 @@
 
 #undef GDK_DISABLE_DEPRECATED
 
+#include <config.h>
+#include "gdkalias.h"
 #include "gdkdisplay.h"
 #include "gdkfont.h"
 #include "gdkinternals.h"
