@@ -24,9 +24,11 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
+#include "gdkalias.h"
 #include "gdk.h"		/* For gdk_flush() */
 #include "gdkimage.h"
 #include "gdkprivate.h"
@@ -215,7 +217,7 @@ allocate_scratch_images (GdkScratchImageInfo *info,
 	  gint j;
 	  
 	  for (j = 0; j < i; j++)
-	    g_object_unref (info->static_image[i]);
+	    g_object_unref (info->static_image[j]);
 	  
 	  return FALSE;
 	}
