@@ -1,6 +1,7 @@
 /* GDK - The GIMP Drawing Kit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
- * Copyright (C) 1998-2002 Tor Lillqvist
+ * Copyright (C) 1998-2004 Tor Lillqvist
+ * Copyright (C) 2000-2004 Hans Breuer
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -689,7 +690,7 @@ gdk_gc_copy (GdkGC *dst_gc,
   GDK_NOTE (GC, g_print ("gdk_gc_copy: %p := %p\n", dst_win32_gc, src_win32_gc));
 
   if (dst_gc->colormap)
-    g_object_unref (G_OBJECT (dst_gc->colormap));
+    g_object_unref (dst_gc->colormap);
 
   if (dst_win32_gc->hcliprgn != NULL)
     DeleteObject (dst_win32_gc->hcliprgn);
@@ -712,7 +713,7 @@ gdk_gc_copy (GdkGC *dst_gc,
   dst_gc->ts_y_origin = src_gc->ts_y_origin;
   dst_gc->colormap = src_gc->colormap;
   if (dst_gc->colormap)
-    g_object_ref (G_OBJECT (dst_gc->colormap));
+    g_object_ref (dst_gc->colormap);
 
   dst_win32_gc->hcliprgn = src_win32_gc->hcliprgn;
   if (dst_win32_gc->hcliprgn)
@@ -1259,3 +1260,23 @@ _gdk_win32_gdkregion_to_hrgn (GdkRegion *region,
 
   return (hrgn);
 }
+
+void
+_gdk_windowing_gc_get_foreground (GdkGC    *gc,
+				  GdkColor *color)
+{
+  GdkGCWin32 *win32_gc;
+  GdkColormap *cmap;
+  
+  g_return_if_fail (GDK_IS_GC_WIN32 (gc));
+
+  win32_gc = GDK_GC_WIN32 (gc);
+
+  color->pixel = win32_gc->foreground;
+  cmap = gdk_gc_get_colormap (gc);
+
+  if (cmap)
+    gdk_colormap_query_color (cmap, win32_gc->foreground, color);
+  else
+    g_warning ("No colormap in _gdk_windowing_gc_get_foreground");
+}
