@@ -53,16 +53,17 @@
 
 static gboolean gdk_synchronize = FALSE;
 
+static gboolean dummy;
+
 GOptionEntry _gdk_windowing_args[] = {
   { "sync", 0, 0, G_OPTION_ARG_NONE, &gdk_synchronize, 
     /* Description of --sync in --help output */              N_("Don't batch GDI requests"), NULL },
   { "no-wintab", 0, 0, G_OPTION_ARG_NONE, &_gdk_input_ignore_wintab, 
-    /* Description of --no-wintab in --help output */         N_("Don't use the Wintab API for tablet support [default]"), NULL },
+    /* Description of --no-wintab in --help output */         N_("Don't use the Wintab API for tablet support"), NULL },
   { "ignore-wintab", 0, 0, G_OPTION_ARG_NONE, &_gdk_input_ignore_wintab, 
     /* Description of --ignore-wintab in --help output */     N_("Same as --no-wintab"), NULL },
-#if 0
-  { "use-wintab", 0, 0, G_OPTION_ARG_NONE, &_gdk_input_ignore_wintab, NULL, NULL },
-#endif
+  { "use-wintab", 0, 0, G_OPTION_ARG_NONE, &dummy,
+    /* Description of --use-wintab in --help output */     N_("Do use the Wintab API [default]"), NULL },
   { "max-colors", 0, 0, G_OPTION_ARG_INT, &_gdk_max_colors, 
     /* Description of --max-colors=COLORS in --help output */ N_("Size of the palette in 8 bit mode"), 
     /* Placeholder in --max-colors=COLORS in --help output */ N_("COLORS") },
@@ -97,13 +98,6 @@ _gdk_windowing_init (void)
   _gdk_app_hmodule = GetModuleHandle (NULL);
   _gdk_display_hdc = CreateDC ("DISPLAY", NULL, NULL, NULL);
   _gdk_root_window = GetDesktopWindow ();
-  _windows_version = GetVersion ();
-
-  if (getenv ("PRETEND_WIN9X"))
-    _windows_version = 0x80000004;
-
-  GDK_NOTE (MISC, g_print ("Windows version: %08x\n", (guint) _windows_version));
-
   _gdk_input_locale = GetKeyboardLayout (0);
   _gdk_input_locale_is_ime = ImmIsIME (_gdk_input_locale);
   GetLocaleInfo (MAKELCID (LOWORD (_gdk_input_locale), SORT_DEFAULT),
@@ -159,7 +153,7 @@ _gdk_win32_gdi_failed (const gchar *where,
   /* On Win9x GDI calls are implemented in 16-bit code and thus
    * don't set the 32-bit error code, sigh.
    */
-  if (IS_WIN_NT ())
+  if (G_WIN32_IS_NT_BASED ())
     _gdk_win32_api_failed (where, line, api);
   else
     _gdk_other_api_failed (where, line, api);
