@@ -158,7 +158,7 @@ _gdk_selection_property_store (GdkWindow *owner,
 	  return;
 	}
       else
-	prop->length = strlen (prop->data + 1);
+	prop->length = strlen (prop->data) + 1;
     }
   else
     {
@@ -179,7 +179,7 @@ _gdk_dropfiles_store (gchar *data)
 
       dropfiles_prop = g_new (GdkSelProp, 1);
       dropfiles_prop->data = data;
-      dropfiles_prop->length = strlen (data);
+      dropfiles_prop->length = strlen (data) + 1;
       dropfiles_prop->format = 8;
       dropfiles_prop->type = _text_uri_list;
     }
@@ -417,7 +417,7 @@ gdk_selection_convert (GdkWindow *requestor,
 	return;
 
       /* Try various formats. First the simplest, CF_UNICODETEXT. */
-      if (IS_WIN_NT () && (hdata = GetClipboardData (CF_UNICODETEXT)) != NULL)
+      if (G_WIN32_IS_NT_BASED () && (hdata = GetClipboardData (CF_UNICODETEXT)) != NULL)
 	{
 	  wchar_t *ptr, *wcs, *p, *q;
 	  guchar *data;
