@@ -43,7 +43,12 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
+#include <config.h>
+#ifdef NEED_XIPROTO_H_FOR_XREPLY
+#include <X11/extensions/XIproto.h>
+#endif
 #include <X11/Xlibint.h>
+#include "gdkalias.h"
 #include "gdkasync.h"
 #include "gdkx.h"
 
