@@ -27,6 +27,7 @@
 #include <string.h>
 
 #include <glib.h>
+#include "gdkalias.h"
 #include "gdkscreen.h"
 #include "gdkscreen-x11.h"
 #include "gdkdisplay.h"
@@ -60,7 +61,7 @@ static gpointer parent_class = NULL;
 static guint signals[LAST_SIGNAL] = { 0 };
 
 GType
-_gdk_screen_x11_get_type ()
+_gdk_screen_x11_get_type (void)
 {
   static GType object_type = 0;
 
@@ -305,6 +306,9 @@ gdk_screen_x11_finalize (GObject *object)
   GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (object);
   /* int i; */
   g_object_unref (screen_x11->root_window);
+
+  if (screen_x11->renderer)
+    g_object_unref (screen_x11->renderer);
   
   /* Visual Part (Need to implement finalize for Visuals for a clean
    * finalize) */
