@@ -181,6 +181,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 
   object_class->finalize = gtk_about_dialog_finalize;
 
+  /**
+   * GtkAboutDialog:name:
+   *
+   * The name of the program. 
+   * If this is not set, it defaults to g_get_application_name().
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_NAME,
 				   g_param_spec_string ("name",
@@ -189,6 +197,13 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:version:
+   *
+   * The version of the program. 
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_VERSION,
 				   g_param_spec_string ("version",
@@ -196,6 +211,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							P_("The version of the program"),
 							NULL,
 							G_PARAM_READWRITE));
+
+  /**
+   * GtkAboutDialog:copyright:
+   *
+   * Copyright information for the program. 
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_COPYRIGHT,
 				   g_param_spec_string ("copyright",
@@ -204,6 +227,16 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 	
+
+  /**
+   * GtkAboutDialog:comments:
+   *
+   * Comments about the program. This string is displayed in a label 
+   * in the main dialog, thus it should be a short explanation of
+   * the main purpose of the program, not a detailed list of features.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_COMMENTS,
 				   g_param_spec_string ("comments",
@@ -211,6 +244,16 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							P_("Comments about the program"),
 							NULL,
 							G_PARAM_READWRITE));
+
+  /**
+   * GtkAboutDialog:license:
+   *
+   * The license of the program. This string is displayed in a 
+   * text view in a secondary dialog, therefore it is fine to use
+   * a long multi-paragraph text.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_LICENSE,
 				   g_param_spec_string ("license",
@@ -219,6 +262,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:website:
+   *
+   * The URL for the link to the website of the program. 
+   * This should be a string starting with "http://.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_WEBSITE,
 				   g_param_spec_string ("website",
@@ -227,6 +278,15 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:website-label:
+   *
+   * The label for the link to the website of the program. If this is not set, 
+   * it defaults to the URL specified in the 
+   * <link linkend="GtkAboutDialog--website">website</link> property.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_WEBSITE_LABEL,
 				   g_param_spec_string ("website_label",
@@ -235,13 +295,32 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:authors:
+   *
+   * The authors of the program, as a %NULL-terminated array of strings.
+   * Each string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_AUTHORS,
 				   g_param_spec_boxed ("authors",
 						       P_("Authors"),
-						       P_("List of authors of the programs"),
+						       P_("List of authors of the program"),
 						       G_TYPE_STRV,
 						       G_PARAM_READWRITE));
+
+  /**
+   * GtkAboutDialog:documenters:
+   *
+   * The people documenting the program, as a %NULL-terminated array of strings.
+   * Each string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_DOCUMENTERS,
 				   g_param_spec_boxed ("documenters",
@@ -250,6 +329,15 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 						       G_TYPE_STRV,
 						       G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:artists:
+   *
+   * The people who contributed artwork to the program, as a %NULL-terminated array of strings.
+   * Each string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_ARTISTS,
 				   g_param_spec_boxed ("artists",
@@ -258,6 +346,16 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 						       G_TYPE_STRV,
 						       G_PARAM_READWRITE));
 
+
+  /**
+   * GtkAboutDialog:translator-credits:
+   *
+   * Credits to the translators. This string should be marked as translatable.
+   * The string may contain email addresses and URLs, which will be displayed
+   * as links, see the introduction for more details.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_TRANSLATOR_CREDITS,
 				   g_param_spec_string ("translator_credits",
@@ -266,6 +364,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							NULL,
 							G_PARAM_READWRITE));
 	
+  /**
+   * GtkAboutDialog:logo:
+   *
+   * A logo for the about box. If this is not set, it defaults to 
+   * gtk_window_get_default_icon_list().
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_LOGO,
 				   g_param_spec_object ("logo",
@@ -274,6 +380,14 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 							GDK_TYPE_PIXBUF,
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkAboutDialog:logo-icon-name:
+   *
+   * A named icon to use as the logo for the about box. This property
+   * overrides the <link linkend="GtkAboutDialog--logo">logo</link> property.
+   *
+   * Since: 2.6
+   */  
   g_object_class_install_property (object_class,
 				   PROP_LOGO_ICON_NAME,
 				   g_param_spec_string ("logo_icon_name",
@@ -310,6 +424,7 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   priv->website = NULL;
   priv->website_label = NULL;
   priv->translator_credits = NULL;
+  priv->license = NULL;
   priv->authors = NULL;
   priv->documenters = NULL;
   priv->artists = NULL;
@@ -446,13 +561,13 @@ gtk_about_dialog_set_property (GObject      *object,
       gtk_about_dialog_set_logo (about, g_value_get_object (value));
       break; 
     case PROP_AUTHORS:
-      gtk_about_dialog_set_authors (about, (gchar**)g_value_get_boxed (value));
+      gtk_about_dialog_set_authors (about, (const gchar**)g_value_get_boxed (value));
       break;
     case PROP_DOCUMENTERS:
-      gtk_about_dialog_set_documenters (about, (gchar**)g_value_get_boxed (value));
+      gtk_about_dialog_set_documenters (about, (const gchar**)g_value_get_boxed (value));
       break;	
     case PROP_ARTISTS:
-      gtk_about_dialog_set_artists (about, (gchar**)g_value_get_boxed (value));
+      gtk_about_dialog_set_artists (about, (const gchar**)g_value_get_boxed (value));
       break;	
     case PROP_TRANSLATOR_CREDITS:
       gtk_about_dialog_set_translator_credits (about, g_value_get_string (value));
@@ -519,7 +634,7 @@ gtk_about_dialog_get_property (GObject    *object,
     case PROP_LOGO_ICON_NAME:
       if (gtk_image_get_storage_type (GTK_IMAGE (priv->logo_image)) == GTK_IMAGE_ICON_NAME)
 	{
-	  gchar *icon_name;
+	  const gchar *icon_name;
 
 	  gtk_image_get_icon_name (GTK_IMAGE (priv->logo_image), &icon_name, NULL);
 	  g_value_set_string (value, icon_name);
@@ -915,6 +1030,7 @@ gtk_about_dialog_set_website (GtkAboutDialog *about,
     {
       priv->website = NULL;
       g_object_set_data (G_OBJECT (priv->website_button), "url", NULL);
+      gtk_widget_hide (priv->website_button);
     }
   g_free (tmp);
 
@@ -1000,7 +1116,7 @@ gtk_about_dialog_set_website_label (GtkAboutDialog *about,
  *
  * Since: 2.6
  **/
-gchar **
+G_CONST_RETURN gchar * G_CONST_RETURN *
 gtk_about_dialog_get_authors (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
@@ -1009,7 +1125,7 @@ gtk_about_dialog_get_authors (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->authors;
+  return (const gchar * const *) priv->authors;
 }
 
 /**
@@ -1024,7 +1140,7 @@ gtk_about_dialog_get_authors (GtkAboutDialog *about)
  **/
 void
 gtk_about_dialog_set_authors (GtkAboutDialog  *about, 
-			      gchar          **authors)
+			      const gchar    **authors)
 {
   GtkAboutDialogPrivate *priv;
   gchar **tmp;
@@ -1034,7 +1150,7 @@ gtk_about_dialog_set_authors (GtkAboutDialog  *about,
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
   tmp = priv->authors;
-  priv->authors = g_strdupv (authors);
+  priv->authors = g_strdupv ((gchar **)authors);
   g_strfreev (tmp);
 
   if (priv->authors != NULL)
@@ -1056,7 +1172,7 @@ gtk_about_dialog_set_authors (GtkAboutDialog  *about,
  *
  * Since: 2.6
  **/
-gchar **
+G_CONST_RETURN gchar * G_CONST_RETURN *
 gtk_about_dialog_get_documenters (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
@@ -1065,7 +1181,7 @@ gtk_about_dialog_get_documenters (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->documenters;
+  return (const gchar * const *)priv->documenters;
 }
 
 /**
@@ -1080,7 +1196,7 @@ gtk_about_dialog_get_documenters (GtkAboutDialog *about)
  **/
 void
 gtk_about_dialog_set_documenters (GtkAboutDialog *about, 
-				  gchar         **documenters)
+				  const gchar   **documenters)
 {
   GtkAboutDialogPrivate *priv;
   gchar **tmp;
@@ -1090,7 +1206,7 @@ gtk_about_dialog_set_documenters (GtkAboutDialog *about,
   priv = (GtkAboutDialogPrivate *)about->private_data;
   
   tmp = priv->documenters;
-  priv->documenters = g_strdupv (documenters);
+  priv->documenters = g_strdupv ((gchar **)documenters);
   g_strfreev (tmp);
 
   if (priv->documenters != NULL)
@@ -1112,7 +1228,7 @@ gtk_about_dialog_set_documenters (GtkAboutDialog *about,
  *
  * Since: 2.6
  **/
-gchar **
+G_CONST_RETURN gchar * G_CONST_RETURN *
 gtk_about_dialog_get_artists (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
@@ -1121,7 +1237,7 @@ gtk_about_dialog_get_artists (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->artists;
+  return (const gchar * const *)priv->artists;
 }
 
 /**
@@ -1136,7 +1252,7 @@ gtk_about_dialog_get_artists (GtkAboutDialog *about)
  **/
 void
 gtk_about_dialog_set_artists (GtkAboutDialog *about, 
-			      gchar         **artists)
+			      const gchar   **artists)
 {
   GtkAboutDialogPrivate *priv;
   gchar **tmp;
@@ -1146,7 +1262,7 @@ gtk_about_dialog_set_artists (GtkAboutDialog *about,
   priv = (GtkAboutDialogPrivate *)about->private_data;
   
   tmp = priv->artists;
-  priv->artists = g_strdupv (artists);
+  priv->artists = g_strdupv ((gchar **)artists);
   g_strfreev (tmp);
 
   if (priv->artists != NULL)
@@ -1324,7 +1440,7 @@ gtk_about_dialog_set_logo (GtkAboutDialog *about,
  * Returns the icon name displayed as logo in the about dialog.
  * 
  * Return value: the icon name displayed as logo. The string is
- *   owned by the about dialog. If you want to keep a reference
+ *   owned by the dialog. If you want to keep a reference
  *   to it, you have to call g_strdup() on it.
  *
  * Since: 2.6
@@ -1333,7 +1449,7 @@ G_CONST_RETURN gchar *
 gtk_about_dialog_get_logo_icon_name (GtkAboutDialog *about)
 {
   GtkAboutDialogPrivate *priv;
-  gchar *icon_name = NULL;
+  const gchar *icon_name = NULL;
   
   g_return_val_if_fail (GTK_IS_ABOUT_DIALOG (about), NULL);
 
@@ -1432,7 +1548,7 @@ create_link_button (GtkWidget *about,
   g_object_set_data_full (G_OBJECT (button), "url", g_strdup (url), g_free);
   set_link_button_text (about, button, text);
   
-  g_signal_connect (G_OBJECT (button), "clicked", callback, data);
+  g_signal_connect (button, "clicked", callback, data);
 
   return button;
 }
@@ -1651,13 +1767,13 @@ add_credits_page (GtkAboutDialog *about,
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (view), 8);
   gtk_text_view_set_right_margin (GTK_TEXT_VIEW (view), 8);
 
-  g_signal_connect (G_OBJECT (view), "key-press-event",
+  g_signal_connect (view, "key-press-event",
                     G_CALLBACK (credits_key_press_event), about);
-  g_signal_connect (G_OBJECT (view), "event-after",
+  g_signal_connect (view, "event-after",
                     G_CALLBACK (credits_event_after), about);
-  g_signal_connect (G_OBJECT (view), "motion-notify-event", 
+  g_signal_connect (view, "motion-notify-event", 
                     G_CALLBACK (credits_motion_notify_event), about);
-  g_signal_connect (G_OBJECT (view), "visibility-notify-event", 
+  g_signal_connect (view, "visibility-notify-event", 
                     G_CALLBACK (credits_visibility_notify_event), about);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
@@ -1895,14 +2011,14 @@ close_cb (GtkAboutDialog *about)
 GtkWidget *
 gtk_about_dialog_new (void)
 {
-  GtkAboutDialog *dialog = GTK_ABOUT_DIALOG (g_object_new (GTK_TYPE_ABOUT_DIALOG, NULL));
+  GtkAboutDialog *dialog = g_object_new (GTK_TYPE_ABOUT_DIALOG, NULL);
 
   /* force defaults */
   gtk_about_dialog_set_name (dialog, NULL);
   gtk_about_dialog_set_logo (dialog, NULL);
 
   /* Close dialog on user response */
-  g_signal_connect (G_OBJECT (dialog),
+  g_signal_connect (dialog,
                     "response",
                     G_CALLBACK (close_cb),
                     NULL);
@@ -1931,7 +2047,7 @@ gtk_about_dialog_set_email_hook (GtkAboutDialogActivateLinkFunc func,
   GtkAboutDialogActivateLinkFunc old;
 
   if (activate_email_hook_destroy != NULL)
-    (* activate_email_hook_destroy) (activate_url_hook_data);
+    (* activate_email_hook_destroy) (activate_email_hook_data);
 
   old = activate_email_hook;
 
@@ -1963,7 +2079,7 @@ gtk_about_dialog_set_url_hook (GtkAboutDialogActivateLinkFunc func,
   GtkAboutDialogActivateLinkFunc old;
 
   if (activate_url_hook_destroy != NULL)
-    (* activate_url_hook_destroy) (activate_email_hook_data);
+    (* activate_url_hook_destroy) (activate_url_hook_data);
 
   old = activate_url_hook;
 
@@ -1981,6 +2097,8 @@ gtk_about_dialog_set_url_hook (GtkAboutDialogActivateLinkFunc func,
  * @Varargs: value of first property, followed by more properties, %NULL-terminated
  *
  * This is a convenience function for showing an application's about box.
+ * The constructed dialog is associated with the parent window and 
+ * reused for future invocations of this function.
  *
  * Since: 2.6
  */
@@ -2002,7 +2120,7 @@ gtk_show_about_dialog (GtkWindow   *parent,
     {
       dialog = gtk_about_dialog_new ();
 
-      g_object_ref (G_OBJECT (dialog));
+      g_object_ref (dialog);
       gtk_object_sink (GTK_OBJECT (dialog));
 
       g_signal_connect (dialog, "delete_event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
@@ -2014,6 +2132,7 @@ gtk_show_about_dialog (GtkWindow   *parent,
       if (parent) 
 	{
 	  gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+	  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 	  g_object_set_data_full (G_OBJECT (parent), "gtk-about-dialog", 
 				  dialog, g_object_unref);
 	}
