@@ -1,5 +1,6 @@
 /* GTK - The GIMP Toolkit
  * Copyright (C) 2000 Red Hat, Inc.
+ * Copyright (C) 2004 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,6 +23,7 @@
 #include <config.h>
 #include <string.h>
 
+#include "gtkalias.h"
 #include "gtkclipboard.h"
 #include "gtkinvisible.h"
 #include "gtkmain.h"
@@ -44,6 +46,7 @@ typedef struct _GtkClipboardClass GtkClipboardClass;
 
 typedef struct _RequestContentsInfo RequestContentsInfo;
 typedef struct _RequestTextInfo RequestTextInfo;
+typedef struct _RequestImageInfo RequestImageInfo;
 typedef struct _RequestTargetsInfo RequestTargetsInfo;
 
 struct _GtkClipboard 
@@ -61,6 +64,16 @@ struct _GtkClipboard
 
   gboolean have_selection;
   GdkDisplay *display;
+
+  GdkAtom *cached_targets;
+  gint     n_cached_targets;
+
+  guint      notify_signal_id;
+  gboolean   storing_selection;
+  GMainLoop *store_loop;
+  guint      store_timeout;
+  gint       n_storable_targets;
+  GdkAtom   *storable_targets;
 };
 
 struct _GtkClipboardClass
@@ -83,27 +96,39 @@ struct _RequestTextInfo
   gpointer user_data;
 };
 
+struct _RequestImageInfo
+{
+  GtkClipboardImageReceivedFunc callback;
+  gpointer user_data;
+};
+
 struct _RequestTargetsInfo
 {
   GtkClipboardTargetsReceivedFunc callback;
   gpointer user_data;
 };
 
-static void gtk_clipboard_class_init (GtkClipboardClass *class);
-static void gtk_clipboard_finalize   (GObject           *object);
+static void gtk_clipboard_class_init   (GtkClipboardClass   *class);
+static void gtk_clipboard_finalize     (GObject             *object);
+static void gtk_clipboard_owner_change (GtkClipboard        *clipboard,
+					GdkEventOwnerChange *event);
+
+static void          clipboard_unset      (GtkClipboard     *clipboard);
+static void          selection_received   (GtkWidget        *widget,
+					   GtkSelectionData *selection_data,
+					   guint             time);
+static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
+					   GdkAtom           selection,
+					   gboolean          only_if_exists);
+static GtkWidget *   get_clipboard_widget (GdkDisplay       *display);
+
 
-static void clipboard_unset          (GtkClipboard      *clipboard);
-static void selection_received       (GtkWidget         *widget,
-				      GtkSelectionData  *selection_data,
-				      guint              time);
-static GtkClipboard *clipboard_peek  (GdkDisplay        *display,
-				      GdkAtom            selection,
-				      gboolean           only_if_exists);
 enum {
   TARGET_STRING,
   TARGET_TEXT,
   TARGET_COMPOUND_TEXT,
-  TARGET_UTF8_STRING
+  TARGET_UTF8_STRING,
+  TARGET_SAVE_TARGETS
 };
 
 static const gchar request_contents_key[] = "gtk-request-contents";
@@ -151,7 +176,7 @@ gtk_clipboard_class_init (GtkClipboardClass *class)
   
   gobject_class->finalize = gtk_clipboard_finalize;
 
-  class->owner_change = NULL;
+  class->owner_change = gtk_clipboard_owner_change;
 
   clipboard_signals[OWNER_CHANGE] =
     g_signal_new ("owner_change",
@@ -167,7 +192,37 @@ gtk_clipboard_class_init (GtkClipboardClass *class)
 static void
 gtk_clipboard_finalize (GObject *object)
 {
-  clipboard_unset (GTK_CLIPBOARD (object));
+  GtkClipboard *clipboard;
+  GtkWidget *clipboard_widget;
+  GSList *clipboards;
+
+  clipboard = GTK_CLIPBOARD (object);
+
+  clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
+  if (g_slist_index (clipboards, clipboard) >= 0)
+    g_warning ("GtkClipboard prematurely finalized");
+
+  clipboard_widget = get_clipboard_widget (clipboard->display);
+  
+  clipboard_unset (clipboard);
+  
+  clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
+  clipboards = g_slist_remove (clipboards, clipboard);
+  g_object_set_data (G_OBJECT (clipboard->display), "gtk-clipboard-list", clipboards);
+
+  if (g_main_loop_is_running (clipboard->store_loop))
+    {
+      g_main_loop_quit (clipboard->store_loop);
+      g_main_loop_unref (clipboard->store_loop);
+    }
+  
+  if (clipboard->store_timeout != 0)
+    g_source_remove (clipboard->store_timeout);
+
+  if (clipboard->notify_signal_id != 0)
+    g_signal_handler_disconnect (clipboard_widget, clipboard->notify_signal_id);
+  
+  g_free (clipboard->storable_targets);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -181,10 +236,9 @@ clipboard_display_closed (GdkDisplay   *display,
 
   clipboards = g_object_get_data (G_OBJECT (display), "gtk-clipboard-list");
   g_object_run_dispose (G_OBJECT (clipboard));
-  g_object_unref (clipboard);
   clipboards = g_slist_remove (clipboards, clipboard);
-  
   g_object_set_data (G_OBJECT (display), "gtk-clipboard-list", clipboards);
+  g_object_unref (clipboard);
 }
 
 /**
@@ -222,8 +276,9 @@ clipboard_display_closed (GdkDisplay   *display,
  * Return value: the appropriate clipboard object. If no
  *             clipboard already exists, a new one will
  *             be created. Once a clipboard object has
- *             been created, it is persistent for all time and
- *             cannot be freed.
+ *             been created, it is persistent and, since
+ *             it is owned by GTK+, must not be freed or
+ *             unrefd.
  *
  * Since: 2.2
  **/
@@ -249,8 +304,9 @@ gtk_clipboard_get_for_display (GdkDisplay *display,
  * Return value: the appropriate clipboard object. If no
  *             clipboard already exists, a new one will
  *             be created. Once a clipboard object has
- *             been created, it is persistent for all time and
- *             cannot be freed.
+ *             been created, it is persistent and, since
+ *             it is owned by GTK+, must not be freed or
+ *             unrefd.
  **/
 GtkClipboard *
 gtk_clipboard_get (GdkAtom selection)
@@ -279,6 +335,7 @@ selection_clear_event_cb (GtkWidget	    *widget,
   if (clipboard)
     {
       clipboard_unset (clipboard);
+
       return TRUE;
     }
 
@@ -591,22 +648,37 @@ clipboard_unset (GtkClipboard *clipboard)
 {
   GtkClipboardClearFunc old_clear_func;
   gpointer old_data;
+  gboolean old_have_owner;
+  gint old_n_storable_targets;
   
   old_clear_func = clipboard->clear_func;
   old_data = clipboard->user_data;
-	  
-  if (clipboard->have_owner)
+  old_have_owner = clipboard->have_owner;
+  old_n_storable_targets = clipboard->n_storable_targets;
+  
+  if (old_have_owner)
     {
       clipboard_remove_owner_notify (clipboard);
       clipboard->have_owner = FALSE;
     }
-  
+
+  clipboard->n_storable_targets = -1;
+  g_free (clipboard->storable_targets);
+  clipboard->storable_targets = NULL;
+      
   clipboard->get_func = NULL;
   clipboard->clear_func = NULL;
   clipboard->user_data = NULL;
   
   if (old_clear_func)
     old_clear_func (clipboard, old_data);
+
+  /* If we've transferred the clipboard data to the manager,
+   * unref the owner
+   */
+  if (old_have_owner &&
+      old_n_storable_targets != -1)
+    g_object_unref (clipboard->user_data);
 }
 
 /**
@@ -647,6 +719,7 @@ text_clear_func (GtkClipboard *clipboard,
   g_free (data);
 }
 
+
 /**
  * gtk_clipboard_set_text:
  * @clipboard: a #GtkClipboard object
@@ -664,23 +737,97 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
 			const gchar  *text,
 			gint          len)
 {
-  static const GtkTargetEntry targets[] = {
-    { "STRING", 0, TARGET_STRING },
-    { "TEXT",   0, TARGET_TEXT }, 
-    { "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT },
-    { "UTF8_STRING", 0, TARGET_UTF8_STRING }
-  };
+  GtkTargetList *list;
+  GList *l;
+  GtkTargetEntry *targets;
+  gint n_targets, i;
 
   g_return_if_fail (clipboard != NULL);
   g_return_if_fail (text != NULL);
+
+  list = gtk_target_list_new (NULL, 0);
+  gtk_target_list_add_text_targets (list, 0);
+
+  n_targets = g_list_length (list->list);
+  targets = g_new0 (GtkTargetEntry, n_targets);
+  for (l = list->list, i = 0; l; l = l->next, i++)
+    {
+      GtkTargetPair *pair = (GtkTargetPair *)l->data;
+      targets[i].target = gdk_atom_name (pair->target);
+    }
   
   if (len < 0)
     len = strlen (text);
   
   gtk_clipboard_set_with_data (clipboard, 
-			       targets, G_N_ELEMENTS (targets),
+			       targets, n_targets,
 			       text_get_func, text_clear_func,
 			       g_strndup (text, len));
+  gtk_clipboard_set_can_store (clipboard, NULL, 0);
+
+  g_free (targets);
+  gtk_target_list_unref (list);
+}
+
+static void 
+pixbuf_get_func (GtkClipboard     *clipboard,
+		 GtkSelectionData *selection_data,
+		 guint             info,
+		 gpointer          data)
+{
+  gtk_selection_data_set_pixbuf (selection_data, data);
+}
+
+static void 
+pixbuf_clear_func (GtkClipboard *clipboard,
+		   gpointer      data)
+{
+  g_object_unref (data);
+}
+
+/**
+ * gtk_clipboard_set_image:
+ * @clipboard: a #GtkClipboard object
+ * @pixbuf:    a #GdkPixbuf 
+ * 
+ * Sets the contents of the clipboard to the given #GdkPixbuf. 
+ * GTK+ will take responsibility for responding for requests 
+ * for the image, and for converting the image into the 
+ * requested format.
+ * 
+ * Since: 2.6
+ **/
+void
+gtk_clipboard_set_image (GtkClipboard *clipboard,
+			  GdkPixbuf    *pixbuf)
+{
+  GtkTargetList *list;
+  GList *l;
+  GtkTargetEntry *targets;
+  gint n_targets, i;
+
+  g_return_if_fail (clipboard != NULL);
+  g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
+
+  list = gtk_target_list_new (NULL, 0);
+  gtk_target_list_add_image_targets (list, 0, TRUE);
+
+  n_targets = g_list_length (list->list);
+  targets = g_new0 (GtkTargetEntry, n_targets);
+  for (l = list->list, i = 0; l; l = l->next, i++)
+    {
+      GtkTargetPair *pair = (GtkTargetPair *)l->data;
+      targets[i].target = gdk_atom_name (pair->target);
+    }
+
+  gtk_clipboard_set_with_data (clipboard, 
+			       targets, n_targets,
+			       pixbuf_get_func, pixbuf_clear_func,
+			       g_object_ref (pixbuf));
+  gtk_clipboard_set_can_store (clipboard, NULL, 0);
+
+  g_free (targets);
+  gtk_target_list_unref (list);
 }
 
 static void
@@ -839,6 +986,83 @@ gtk_clipboard_request_text (GtkClipboard                *clipboard,
 				  info);
 }
 
+static void 
+request_image_received_func (GtkClipboard     *clipboard,
+			     GtkSelectionData *selection_data,
+			     gpointer          data)
+{
+  RequestImageInfo *info = data;
+  GdkPixbuf *result = NULL;
+
+  result = gtk_selection_data_get_pixbuf (selection_data);
+
+  if (!result)
+    {
+      /* If we asked for image/png and didn't get it, try image/jpeg;
+       * if we asked for image/jpeg and didn't get it, try image/gif;
+       * If we asked for anything else and didn't get it, give up.
+       */
+      if (selection_data->target == gdk_atom_intern ("image/png", FALSE))
+	{
+	  gtk_clipboard_request_contents (clipboard,
+					  gdk_atom_intern ("image/jpeg", FALSE), 
+					  request_image_received_func, info);
+	  return;
+	}
+      else if (selection_data->target == gdk_atom_intern ("image/jpeg", FALSE))
+	{
+	  gtk_clipboard_request_contents (clipboard,
+					  gdk_atom_intern ("image/gif", FALSE), 
+					  request_text_received_func, info);
+	  return;
+	}
+    }
+
+  info->callback (clipboard, result, info->user_data);
+  g_free (info);
+  g_object_unref (result);
+}
+
+/**
+ * gtk_clipboard_request_image:
+ * @clipboard: a #GtkClipboard
+ * @callback:  a function to call when the image is received,
+ *             or the retrieval fails. (It will always be called
+ *             one way or the other.)
+ * @user_data: user data to pass to @callback.
+ * 
+ * Requests the contents of the clipboard as image. When the image is
+ * later received, it will be converted to a #GdkPixbuf, and
+ * @callback will be called. 
+ *
+ * The @pixbuf parameter to @callback will contain the resulting 
+ * #GdkPixbuf if the request succeeded, or %NULL if it failed. This 
+ * could happen for various reasons, in particular if the clipboard 
+ * was empty or if the contents of the clipboard could not be 
+ * converted into an image.
+ *
+ * Since: 2.6
+ **/
+void 
+gtk_clipboard_request_image (GtkClipboard                  *clipboard,
+			     GtkClipboardImageReceivedFunc  callback,
+			     gpointer                       user_data)
+{
+  RequestImageInfo *info;
+  
+  g_return_if_fail (clipboard != NULL);
+  g_return_if_fail (callback != NULL);
+  
+  info = g_new (RequestImageInfo, 1);
+  info->callback = callback;
+  info->user_data = user_data;
+
+  gtk_clipboard_request_contents (clipboard, 
+				  gdk_atom_intern ("image/png", FALSE),
+				  request_image_received_func,
+				  info);
+}
+
 static void 
 request_targets_received_func (GtkClipboard     *clipboard,
 			       GtkSelectionData *selection_data,
@@ -881,6 +1105,14 @@ gtk_clipboard_request_targets (GtkClipboard                *clipboard,
   
   g_return_if_fail (clipboard != NULL);
   g_return_if_fail (callback != NULL);
+
+  /* If the display supports change notification we cache targets */
+  if (gdk_display_supports_selection_notification (gtk_clipboard_get_display (clipboard)) &&
+      clipboard->n_cached_targets != -1)
+    {
+      (* callback) (clipboard, clipboard->cached_targets, clipboard->n_cached_targets, user_data);
+      return;
+    }
   
   info = g_new (RequestTargetsInfo, 1);
   info->callback = callback;
@@ -964,7 +1196,6 @@ clipboard_text_received_func (GtkClipboard *clipboard,
   g_main_loop_quit (results->loop);
 }
 
-
 /**
  * gtk_clipboard_wait_for_text:
  * @clipboard: a #GtkClipboard
@@ -986,7 +1217,6 @@ gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
 {
   WaitResults results;
 
-  g_return_val_if_fail (clipboard != NULL, NULL);
   g_return_val_if_fail (clipboard != NULL, NULL);
   
   results.data = NULL;
@@ -1007,6 +1237,64 @@ gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
 
   return results.data;
 }
+
+static void 
+clipboard_image_received_func (GtkClipboard *clipboard,
+			       GdkPixbuf    *pixbuf,
+			       gpointer      data)
+{
+  WaitResults *results = data;
+
+  if (pixbuf)
+    results->data = g_object_ref (pixbuf);
+
+  g_main_loop_quit (results->loop);
+}
+
+/**
+ * gtk_clipboard_wait_for_image:
+ * @clipboard: a #GtkClipboard
+ * 
+ * Requests the contents of the clipboard as image and converts
+ * the result to a #GdkPixbuf. This function waits for
+ * the data to be received using the main loop, so events,
+ * timeouts, etc, may be dispatched during the wait.
+ * 
+ * Return value: a newly-allocated #GdkPixbuf object which must
+ *               be disposed with g_object_unref(), or %NULL if 
+ *               retrieving the selection data failed. (This 
+ *               could happen for various reasons, in particular 
+ *               if the clipboard was empty or if the contents of 
+ *               the clipboard could not be converted into an image.)
+ *
+ * Since: 2.6
+ **/
+GdkPixbuf *
+gtk_clipboard_wait_for_image (GtkClipboard *clipboard)
+{
+  WaitResults results;
+
+  g_return_val_if_fail (clipboard != NULL, NULL);
+  
+  results.data = NULL;
+  results.loop = g_main_loop_new (NULL, TRUE);
+
+  gtk_clipboard_request_image (clipboard,
+			       clipboard_image_received_func,
+			       &results);
+
+  if (g_main_loop_is_running (results.loop))
+    {
+      GDK_THREADS_LEAVE ();
+      g_main_loop_run (results.loop);
+      GDK_THREADS_ENTER ();
+    }
+
+  g_main_loop_unref (results.loop);
+
+  return results.data;
+}
+
 /**
  * gtk_clipboard_get_display:
  * @clipboard: a #GtkClipboard
@@ -1031,10 +1319,9 @@ gtk_clipboard_get_display (GtkClipboard *clipboard)
  * 
  * Test to see if there is text available to be pasted
  * This is done by requesting the TARGETS atom and checking
- * if it contains any of the names: STRING, TEXT, COMPOUND_TEXT,
- * UTF8_STRING. This function waits for the data to be received
- * using the main loop, so events, timeouts, etc, may be dispatched
- * during the wait.
+ * if it contains any of the supported text targets. This function 
+ * waits for the data to be received using the main loop, so events, 
+ * timeouts, etc, may be dispatched during the wait.
  *
  * This function is a little faster than calling
  * gtk_clipboard_wait_for_text() since it doesn't need to retrieve
@@ -1058,6 +1345,41 @@ gtk_clipboard_wait_is_text_available (GtkClipboard *clipboard)
   return result;
 }
 
+/**
+ * gtk_clipboard_wait_is_image_available:
+ * @clipboard: a #GtkClipboard
+ * 
+ * Test to see if there is an image available to be pasted
+ * This is done by requesting the TARGETS atom and checking
+ * if it contains any of the supported image targets. This function 
+ * waits for the data to be received using the main loop, so events, 
+ * timeouts, etc, may be dispatched during the wait.
+ *
+ * This function is a little faster than calling
+ * gtk_clipboard_wait_for_image() since it doesn't need to retrieve
+ * the actual image data.
+ * 
+ * Return value: %TRUE is there is an image available, %FALSE otherwise.
+ *
+ * Since: 2.6
+ **/
+gboolean
+gtk_clipboard_wait_is_image_available (GtkClipboard *clipboard)
+{
+  GtkSelectionData *data;
+  gboolean result = FALSE;
+
+  data = gtk_clipboard_wait_for_contents (clipboard, 
+					  gdk_atom_intern ("TARGETS", FALSE));
+  if (data)
+    {
+      result = gtk_selection_data_targets_include_image (data, FALSE);
+      gtk_selection_data_free (data);
+    }
+
+  return result;
+}
+
 /**
  * gtk_clipboard_wait_for_targets
  * @clipboard: a #GtkClipboard
@@ -1086,8 +1408,20 @@ gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
   
   g_return_val_if_fail (clipboard != NULL, FALSE);
 
-  /* TODO: see http://bugzilla.gnome.org/show_bug.cgi?id=101774 with regard to XFIXES */
+  /* If the display supports change notification we cache targets */
+  if (gdk_display_supports_selection_notification (gtk_clipboard_get_display (clipboard)) &&
+      clipboard->n_cached_targets != -1)
+    {
+      if (n_targets)
+ 	*n_targets = clipboard->n_cached_targets;
+ 
+      if (targets)
+ 	*targets = g_memdup (clipboard->cached_targets,
+ 			     clipboard->n_cached_targets * sizeof (GdkAtom));
 
+       return TRUE;
+    }
+  
   if (n_targets)
     *n_targets = 0;
       
@@ -1098,7 +1432,26 @@ gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
 
   if (data)
     {
-      result = gtk_selection_data_get_targets (data, targets, n_targets);
+      GdkAtom *tmp_targets;
+      gint tmp_n_targets;
+       
+      result = gtk_selection_data_get_targets (data, &tmp_targets, &tmp_n_targets);
+ 
+      if (gdk_display_supports_selection_notification (gtk_clipboard_get_display (clipboard)))
+ 	{
+ 	  clipboard->n_cached_targets = tmp_n_targets;
+ 	  clipboard->cached_targets = g_memdup (tmp_targets,
+ 						tmp_n_targets * sizeof (GdkAtom));
+ 	}
+ 
+      if (n_targets)
+ 	*n_targets = tmp_n_targets;
+ 
+      if (targets)
+ 	*targets = tmp_targets;
+      else
+ 	g_free (tmp_targets);
+      
       gtk_selection_data_free (data);
     }
 
@@ -1134,6 +1487,8 @@ clipboard_peek (GdkDisplay *display,
       clipboard = g_object_new (GTK_TYPE_CLIPBOARD, NULL);
       clipboard->selection = selection;
       clipboard->display = display;
+      clipboard->n_cached_targets = -1;
+      clipboard->n_storable_targets = -1;
       clipboards = g_slist_prepend (clipboards, clipboard);
       g_object_set_data (G_OBJECT (display), "gtk-clipboard-list", clipboards);
       g_signal_connect (display, "closed",
@@ -1144,6 +1499,57 @@ clipboard_peek (GdkDisplay *display,
   return clipboard;
 }
 
+static void
+gtk_clipboard_owner_change (GtkClipboard        *clipboard,
+			    GdkEventOwnerChange *event)
+{
+  if (clipboard->n_cached_targets != -1)
+    {
+      clipboard->n_cached_targets = -1;
+      g_free (clipboard->cached_targets);
+    }
+}
+
+/**
+ * gtk_clipboard_wait_is_target_available:
+ * @clipboard: a #GtkClipboard
+ * @target:    A #GdkAtom indicating which target to look for.
+ *
+ * Checks if a clipboard supports pasting data of a given type. This
+ * function can be used to determine if a "Paste" menu item should be
+ * insensitive or not.
+ *
+ * If you want to see if there's text available on the clipboard, use
+ * gtk_clipboard_wait_is_text_available () instead.
+ *
+ * Return value: %TRUE if the target is available, %FALSE otherwise.
+ *
+ * Since: 2.6
+ */
+gboolean
+gtk_clipboard_wait_is_target_available (GtkClipboard *clipboard,
+					GdkAtom       target)
+{
+  GdkAtom *targets;
+  gint i, n_targets;
+  gboolean retval = FALSE;
+    
+  if (!gtk_clipboard_wait_for_targets (clipboard, &targets, &n_targets))
+    return FALSE;
+
+  for (i = 0; i < n_targets; i++)
+    {
+      if (targets[i] == target)
+	{
+	  retval = TRUE;
+	  break;
+	}
+    }
+
+  g_free (targets);
+  
+  return retval;
+}
 
 /**
  * _gtk_clipboard_handle_event:
@@ -1167,3 +1573,161 @@ _gtk_clipboard_handle_event (GdkEventOwnerChange *event)
 		   clipboard_signals[OWNER_CHANGE], 0, event, NULL);
 }
 
+static gboolean
+gtk_clipboard_store_timeout (GtkClipboard *clipboard)
+{
+  g_main_loop_quit (clipboard->store_loop);
+  
+  return FALSE;
+}
+
+/**
+ * gtk_clipboard_set_can_store:
+ * @clipboard: a #GtkClipboard
+ * @targets: array containing information about which forms should be stored
+ *           or %NULL to indicate that all forms should be stored.
+ * @n_targets: number of elements in @targets
+ *
+ * Hints that the clipboard data should be stored somewhere when the
+ * application exits or when gtk_clipboard_store () is called.
+ *
+ * This value is reset when the clipboard owner changes.
+ * Where the clipboard data is stored is platform dependent,
+ * see gdk_display_store_clipboard () for more information.
+ * 
+ * Since: 2.6
+ */
+void
+gtk_clipboard_set_can_store (GtkClipboard         *clipboard,
+ 			     const GtkTargetEntry *targets,
+ 			     gint                  n_targets)
+{
+  GtkWidget *clipboard_widget;
+  int i;
+  static const GtkTargetEntry save_targets[] = {
+    { "SAVE_TARGETS", 0, TARGET_SAVE_TARGETS }
+  };
+  
+  g_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
+
+  if (clipboard->selection != GDK_SELECTION_CLIPBOARD)
+    return;
+  
+  g_free (clipboard->storable_targets);
+  
+  clipboard_widget = get_clipboard_widget (clipboard->display);
+
+  /* n_storable_targets being -1 means that
+   * gtk_clipboard_set_can_store hasn't been called since the
+   * clipboard owner changed. We only want to add SAVE_TARGETS and 
+   * ref the owner once , so we do that here
+   */  
+  if (clipboard->n_storable_targets == -1)
+    {
+      gtk_selection_add_targets (clipboard_widget, clipboard->selection,
+				 save_targets, 1);
+
+      /* Ref the owner so it won't go away */
+      if (clipboard->have_owner)
+	g_object_ref (clipboard->user_data);
+    }
+  
+  clipboard->n_storable_targets = n_targets;
+  clipboard->storable_targets = g_new (GdkAtom, n_targets);
+  for (i = 0; i < n_targets; i++)
+    clipboard->storable_targets[i] = gdk_atom_intern (targets[i].target, FALSE);
+}
+
+static gboolean
+gtk_clipboard_selection_notify (GtkWidget         *widget,
+				GdkEventSelection *event,
+				GtkClipboard      *clipboard)
+{
+  if (event->selection == gdk_atom_intern ("CLIPBOARD_MANAGER", FALSE) &&
+      clipboard->storing_selection)
+    g_main_loop_quit (clipboard->store_loop);
+
+  return FALSE;
+}
+
+/**
+ * gtk_clipboard_store:
+ * @clipboard: a #GtkClipboard
+ *
+ * Stores the current clipboard data somewhere so that it will stay
+ * around after the application has quit.
+ *
+ * Since: 2.6
+ */
+void
+gtk_clipboard_store (GtkClipboard *clipboard)
+{
+  GtkWidget *clipboard_widget;
+
+  g_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
+
+  if (clipboard->n_storable_targets < 0)
+    return;
+  
+  if (!gdk_display_supports_clipboard_persistence (clipboard->display))
+    return;
+
+  clipboard_widget = get_clipboard_widget (clipboard->display);
+  clipboard->notify_signal_id = g_signal_connect (clipboard_widget, "selection_notify_event",
+						  G_CALLBACK (gtk_clipboard_selection_notify), clipboard);
+  
+  gdk_display_store_clipboard (clipboard->display,
+			       clipboard_widget->window,
+			       clipboard_get_timestamp (clipboard),
+			       clipboard->storable_targets,
+			       clipboard->n_storable_targets);
+
+  clipboard->storing_selection = TRUE;
+
+  clipboard->store_loop = g_main_loop_new (NULL, TRUE);
+  clipboard->store_timeout = g_timeout_add (10000, (GSourceFunc) gtk_clipboard_store_timeout, clipboard);
+
+  if (g_main_loop_is_running (clipboard->store_loop))
+    {
+      GDK_THREADS_LEAVE ();
+      g_main_loop_run (clipboard->store_loop);
+      GDK_THREADS_ENTER ();
+    }
+  
+  g_main_loop_unref (clipboard->store_loop);
+  clipboard->store_loop = NULL;
+  
+  g_source_remove (clipboard->store_timeout);
+  clipboard->store_timeout = 0;
+  g_signal_handler_disconnect (clipboard_widget, clipboard->notify_signal_id);
+  clipboard->notify_signal_id = 0;
+  
+  clipboard->storing_selection = FALSE;
+}
+
+/* Stores all clipboard selections on all displays, called from
+ * gtk_main_quit ().
+ */
+void
+_gtk_clipboard_store_all (void)
+{
+  GtkClipboard *clipboard;
+  GSList *displays, *list;
+  
+  displays = gdk_display_manager_list_displays (gdk_display_manager_get ());
+
+  list = displays;
+  while (list)
+    {
+      GdkDisplay *display = list->data;
+
+      clipboard = clipboard_peek (display, GDK_SELECTION_CLIPBOARD, TRUE);
+
+      if (clipboard)
+	gtk_clipboard_store (clipboard);
+      
+      list = list->next;
+    }
+  g_slist_free (displays);
+  
+}
