@@ -24,10 +24,12 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdarg.h>
 #include <string.h>
 #include <stdlib.h>
 
+#include "gtkalias.h"
 #include "gtkcontainer.h"
 #include "gtkprivate.h"
 #include "gtkmain.h"
@@ -200,16 +202,16 @@ gtk_container_class_init (GtkContainerClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_RESIZE_MODE,
                                    g_param_spec_enum ("resize_mode",
-                                                      _("Resize mode"),
-                                                      _("Specify how resize events are handled"),
+                                                      P_("Resize mode"),
+                                                      P_("Specify how resize events are handled"),
                                                       GTK_TYPE_RESIZE_MODE,
                                                       GTK_RESIZE_PARENT,
                                                       G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_BORDER_WIDTH,
                                    g_param_spec_uint ("border_width",
-                                                      _("Border width"),
-                                                      _("The width of the empty border outside the containers children"),
+                                                      P_("Border width"),
+                                                      P_("The width of the empty border outside the containers children"),
 						      0,
 						      G_MAXINT,
 						      0,
@@ -217,8 +219,8 @@ gtk_container_class_init (GtkContainerClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_CHILD,
                                    g_param_spec_object ("child",
-                                                      _("Child"),
-                                                      _("Can be used to add a new child to the container"),
+                                                      P_("Child"),
+                                                      P_("Can be used to add a new child to the container"),
                                                       GTK_TYPE_WIDGET,
 						      G_PARAM_WRITABLE));
   container_signals[ADD] =
@@ -1130,7 +1132,7 @@ _gtk_container_queue_resize (GtkContainer *container)
   widget = GTK_WIDGET (container);
   resize_container = gtk_container_get_resize_container (container);
   
-  while (!GTK_WIDGET_ALLOC_NEEDED (widget) || !GTK_WIDGET_REQUEST_NEEDED (widget))
+  while (TRUE)
     {
       GTK_PRIVATE_SET_FLAG (widget, GTK_ALLOC_NEEDED);
       GTK_PRIVATE_SET_FLAG (widget, GTK_REQUEST_NEEDED);
@@ -1459,8 +1461,7 @@ gtk_container_real_set_focus_child (GtkContainer     *container,
 				    GtkWidget        *child)
 {
   g_return_if_fail (GTK_IS_CONTAINER (container));
-  if (child)
-    g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
   if (child != container->focus_child)
     {
@@ -1476,21 +1477,35 @@ gtk_container_real_set_focus_child (GtkContainer     *container,
    */
   if (container->focus_child)
     {
-      GtkAdjustment *adjustment;
-      
-      adjustment = g_object_get_qdata (G_OBJECT (container), vadjustment_key_id);
-      if (adjustment)
-	gtk_adjustment_clamp_page (adjustment,
-				   container->focus_child->allocation.y,
-				   (container->focus_child->allocation.y +
-				    container->focus_child->allocation.height));
+      GtkAdjustment *hadj;
+      GtkAdjustment *vadj;
+      GtkWidget *focus_child;
+      gint x, y;
 
-      adjustment = g_object_get_qdata (G_OBJECT (container), hadjustment_key_id);
-      if (adjustment)
-	gtk_adjustment_clamp_page (adjustment,
-				   container->focus_child->allocation.x,
-				   (container->focus_child->allocation.x +
-				    container->focus_child->allocation.width));
+      hadj = g_object_get_qdata (G_OBJECT (container), hadjustment_key_id);   
+      vadj = g_object_get_qdata (G_OBJECT (container), vadjustment_key_id);
+      if (hadj || vadj) 
+	{
+
+	  focus_child = container->focus_child;
+	  while (GTK_IS_CONTAINER (focus_child) && 
+		 GTK_CONTAINER (focus_child)->focus_child)
+	    {
+	      focus_child = GTK_CONTAINER (focus_child)->focus_child;
+	    }
+	  
+	  gtk_widget_translate_coordinates (focus_child, container->focus_child, 
+					    0, 0, &x, &y);
+
+	   x += container->focus_child->allocation.x;
+	   y += container->focus_child->allocation.y;
+	  
+	  if (vadj)
+	    gtk_adjustment_clamp_page (vadj, y, y + focus_child->allocation.height);
+	  
+	  if (hadj)
+	    gtk_adjustment_clamp_page (hadj, x, x + focus_child->allocation.width);
+	}
     }
 }
 
@@ -2190,6 +2205,21 @@ gtk_container_unset_focus_chain (GtkContainer  *container)
     }
 }
 
+/**
+ * gtk_container_set_focus_vadjustment:
+ * @container: a #GtkContainer
+ * @adjustment: an adjustment which should be adjusted when the focus is moved among the
+ *   descendents of @container
+ * 
+ * Hooks up an adjustment to focus handling in a container, so when a child of the 
+ * container is focused, the adjustment is scrolled to show that widget. This function
+ * sets the vertical alignment. See gtk_scrolled_window_get_vadjustment() for a typical
+ * way of obtaining the adjustment and gtk_container_set_focus_hadjustment() for setting
+ * the horizontal adjustment.
+ *
+ * The adjustments have to be in pixel units and in the same coordinate system as the 
+ * allocation for immediate children of the container. 
+ */
 void
 gtk_container_set_focus_vadjustment (GtkContainer  *container,
 				     GtkAdjustment *adjustment)
@@ -2229,6 +2259,21 @@ gtk_container_get_focus_vadjustment (GtkContainer *container)
   return vadjustment;
 }
 
+/**
+ * gtk_container_set_focus_hadjustment:
+ * @container: a #GtkContainer
+ * @adjustment: an adjustment which should be adjusted when the focus is moved among the
+ *   descendents of @container
+ * 
+ * Hooks up an adjustment to focus handling in a container, so when a child of the 
+ * container is focused, the adjustment is scrolled to show that widget. This function
+ * sets the horizontal alignment. See gtk_scrolled_window_get_hadjustment() for a typical
+ * way of obtaining the adjustment and gtk_container_set_focus_vadjustment() for setting
+ * the vertical adjustment.
+ *
+ * The adjustments have to be in pixel units and in the same coordinate system as the 
+ * allocation for immediate children of the container. 
+ */
 void
 gtk_container_set_focus_hadjustment (GtkContainer  *container,
 				     GtkAdjustment *adjustment)
