@@ -93,6 +93,7 @@ struct _GtkEntry
   
   guint        select_words : 1;
   guint        select_lines : 1;
+  guint        resolved_dir : 4; /* PangoDirection */
   guint   button;
   guint   blink_timeout;
   guint   recompute_idle;
@@ -136,6 +137,7 @@ struct _GtkEntryClass
   void (* delete_from_cursor) (GtkEntry       *entry,
 			       GtkDeleteType   type,
 			       gint            count);
+  void (* backspace)          (GtkEntry       *entry);
   void (* cut_clipboard)      (GtkEntry       *entry);
   void (* copy_clipboard)     (GtkEntry       *entry);
   void (* paste_clipboard)    (GtkEntry       *entry);
@@ -145,7 +147,6 @@ struct _GtkEntryClass
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
-  void (*_gtk_reserved4) (void);
 };
 
 GType      gtk_entry_get_type       		(void) G_GNUC_CONST;
@@ -173,7 +174,7 @@ gint       gtk_entry_get_width_chars            (GtkEntry      *entry);
 
 /* Somewhat more convenient than the GtkEditable generic functions
  */
-void                  gtk_entry_set_text        (GtkEntry      *entry,
+void       gtk_entry_set_text                   (GtkEntry      *entry,
                                                  const gchar   *text);
 /* returns a reference to the text */
 G_CONST_RETURN gchar* gtk_entry_get_text        (GtkEntry      *entry);
@@ -182,11 +183,20 @@ PangoLayout* gtk_entry_get_layout               (GtkEntry      *entry);
 void         gtk_entry_get_layout_offsets       (GtkEntry      *entry,
                                                  gint          *x,
                                                  gint          *y);
+void       gtk_entry_set_alignment              (GtkEntry      *entry,
+                                                 gfloat         xalign);
+gfloat     gtk_entry_get_alignment              (GtkEntry      *entry);
 
 void                gtk_entry_set_completion (GtkEntry           *entry,
                                               GtkEntryCompletion *completion);
 GtkEntryCompletion *gtk_entry_get_completion (GtkEntry           *entry);
 
+gint       gtk_entry_layout_index_to_text_index (GtkEntry      *entry,
+                                                 gint           layout_index);
+gint       gtk_entry_text_index_to_layout_index (GtkEntry      *entry,
+                                                 gint           text_index);
+
+
 /* Deprecated compatibility functions
  */
 
@@ -205,6 +215,12 @@ void       gtk_entry_set_editable   		(GtkEntry      *entry,
 						 gboolean       editable);
 #endif /* GTK_DISABLE_DEPRECATED */
 
+/* private */
+void      _gtk_entry_get_borders                (GtkEntry *entry,
+						 gint     *xborder,
+						 gint     *yborder);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
