@@ -21,6 +21,7 @@
 #include <config.h>
 #include <string.h>
 
+#include "gtkalias.h"
 #include "gtkcelllayout.h"
 #include "gtkcellrenderertext.h"
 #include "gtkentry.h"
@@ -42,6 +43,8 @@ struct _GtkFileChooserEntry
 {
   GtkEntry parent_instance;
 
+  GtkFileChooserAction action;
+
   GtkFileSystem *file_system;
   GtkFilePath *base_folder;
   GtkFilePath *current_folder_path;
@@ -56,6 +59,7 @@ struct _GtkFileChooserEntry
 
   guint has_completion : 1;
   guint in_change      : 1;
+  guint eat_tabs       : 1;
 };
 
 enum
@@ -173,6 +177,7 @@ gtk_file_chooser_entry_init (GtkFileChooserEntry *chooser_entry)
   GtkCellRenderer *cell;
 
   comp = gtk_entry_completion_new ();
+
   gtk_entry_completion_set_match_func (comp,
 				       completion_match_func,
 				       chooser_entry,
@@ -386,15 +391,17 @@ check_completion_callback (GtkFileChooserEntry *chooser_entry)
   GtkFilePath *unique_path = NULL;
   gboolean valid;
 
+  GDK_THREADS_ENTER ();
+
   g_assert (chooser_entry->file_part);
 
   chooser_entry->check_completion_idle = NULL;
 
   if (strcmp (chooser_entry->file_part, "") == 0)
-    return FALSE;
+    goto done;
 
   if (chooser_entry->completion_store == NULL)
-    return FALSE;
+    goto done;
 
   valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (chooser_entry->completion_store),
 					 &iter);
@@ -449,6 +456,19 @@ check_completion_callback (GtkFileChooserEntry *chooser_entry)
       gtk_file_path_free (unique_path);
     }
 
+  switch (chooser_entry->action)
+    {
+    case GTK_FILE_CHOOSER_ACTION_SAVE:
+    case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+      if (common_prefix && !g_str_has_suffix (common_prefix, "/"))
+	{
+	  g_free (common_prefix);
+	  common_prefix = NULL;
+	}
+      break;
+    default: ;
+    }
+
   if (common_prefix)
     {
       gint file_part_len;
@@ -479,6 +499,10 @@ check_completion_callback (GtkFileChooserEntry *chooser_entry)
       g_free (common_prefix);
     }
 
+ done:
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -560,17 +584,19 @@ load_directory_callback (GtkFileChooserEntry *chooser_entry)
 {
   GSList *child_paths = NULL;
 
+  GDK_THREADS_ENTER ();
+
   chooser_entry->load_directory_idle = NULL;
 
   /* guard against bogus settings*/
   if (chooser_entry->current_folder_path == NULL ||
       chooser_entry->file_system == NULL)
-    return FALSE;
+    goto done;
 
   if (chooser_entry->current_folder != NULL)
     {
       g_warning ("idle activate multiple times without clearing the folder object first.");
-      return FALSE;
+      goto done;
     }
   g_assert (chooser_entry->completion_store == NULL);
 
@@ -582,7 +608,7 @@ load_directory_callback (GtkFileChooserEntry *chooser_entry)
 
   /* There is no folder by that name */
   if (!chooser_entry->current_folder)
-    return FALSE;
+    goto done;
   g_signal_connect (chooser_entry->current_folder, "files-added",
 		    G_CALLBACK (files_added_cb), chooser_entry);
   g_signal_connect (chooser_entry->current_folder, "files-removed",
@@ -607,6 +633,11 @@ load_directory_callback (GtkFileChooserEntry *chooser_entry)
 
   gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (chooser_entry)),
 				  GTK_TREE_MODEL (chooser_entry->completion_store));
+
+ done:
+  
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -632,6 +663,9 @@ gtk_file_chooser_entry_focus (GtkWidget        *widget,
   GdkModifierType state;
   gboolean control_pressed = FALSE;
 
+  if (!chooser_entry->eat_tabs)
+    return GTK_WIDGET_CLASS (parent_class)->focus (widget, direction);
+
   if (gtk_get_current_event_state (&state))
     {
       if ((state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
@@ -644,11 +678,16 @@ gtk_file_chooser_entry_focus (GtkWidget        *widget,
       (GTK_WIDGET_HAS_FOCUS (widget)) &&
       (! control_pressed))
     {
+      gint pos = 0;
+
       if (chooser_entry->has_completion)
-	{
-	  gtk_editable_set_position (GTK_EDITABLE (widget),
-				     GTK_ENTRY (widget)->text_length);
-	}
+	gtk_editable_set_position (GTK_EDITABLE (widget),
+				   GTK_ENTRY (widget)->text_length);
+      /* Trigger the completion window to pop up again by a 
+       * zero-length insertion, a bit of a hack.
+       */
+      gtk_editable_insert_text (GTK_EDITABLE (widget), "", -1, &pos);
+
       return TRUE;
     }
   else
@@ -698,10 +737,7 @@ gtk_file_chooser_entry_maybe_update_directory (GtkFileChooserEntry *chooser_entr
 	    }
 	  if (chooser_entry->completion_store)
 	    {
-	      gtk_list_store_clear (GTK_LIST_STORE (chooser_entry->completion_store));
-	      /* FIXME: Uncomment this line and get rid of the _clear above
-	       * after #137211 is fixed */
-	      /* gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (chooser_entry)), NULL);*/
+	      gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (chooser_entry)), NULL);
 	      g_object_unref (chooser_entry->completion_store);
 	      chooser_entry->completion_store = NULL;
 	    }
@@ -788,6 +824,7 @@ clear_completion_callback (GtkFileChooserEntry *chooser_entry,
 
 /**
  * _gtk_file_chooser_entry_new:
+ * @eat_tabs: If %FALSE, allow focus navigation with the tab key.
  *
  * Creates a new #GtkFileChooserEntry object. #GtkFileChooserEntry
  * is an internal implementation widget for the GTK+ file chooser
@@ -797,9 +834,14 @@ clear_completion_callback (GtkFileChooserEntry *chooser_entry,
  * Return value: the newly created #GtkFileChooserEntry
  **/
 GtkWidget *
-_gtk_file_chooser_entry_new (void)
+_gtk_file_chooser_entry_new (gboolean eat_tabs)
 {
-  return g_object_new (GTK_TYPE_FILE_CHOOSER_ENTRY, NULL);
+  GtkFileChooserEntry *chooser_entry;
+
+  chooser_entry = g_object_new (GTK_TYPE_FILE_CHOOSER_ENTRY, NULL);
+  chooser_entry->eat_tabs = (eat_tabs != FALSE);
+
+  return GTK_WIDGET (chooser_entry);
 }
 
 /**
@@ -841,6 +883,7 @@ _gtk_file_chooser_entry_set_base_folder (GtkFileChooserEntry *chooser_entry,
 
   chooser_entry->base_folder = gtk_file_path_copy (path);
 
+  gtk_file_chooser_entry_changed (GTK_EDITABLE (chooser_entry));
   gtk_editable_select_region (GTK_EDITABLE (chooser_entry), 0, -1);
 }
 
@@ -862,6 +905,11 @@ _gtk_file_chooser_entry_set_base_folder (GtkFileChooserEntry *chooser_entry,
 const GtkFilePath *
 _gtk_file_chooser_entry_get_current_folder (GtkFileChooserEntry *chooser_entry)
 {
+  if (chooser_entry->has_completion)
+    {
+      gtk_editable_set_position (GTK_EDITABLE (chooser_entry),
+				 GTK_ENTRY (chooser_entry)->text_length);
+    }
   return chooser_entry->current_folder_path;
 }
 
@@ -880,6 +928,11 @@ _gtk_file_chooser_entry_get_current_folder (GtkFileChooserEntry *chooser_entry)
 const gchar *
 _gtk_file_chooser_entry_get_file_part (GtkFileChooserEntry *chooser_entry)
 {
+  if (chooser_entry->has_completion)
+    {
+      gtk_editable_set_position (GTK_EDITABLE (chooser_entry),
+				 GTK_ENTRY (chooser_entry)->text_length);
+    }
   return chooser_entry->file_part;
 }
 
@@ -898,3 +951,43 @@ _gtk_file_chooser_entry_set_file_part (GtkFileChooserEntry *chooser_entry,
 
   gtk_entry_set_text (GTK_ENTRY (chooser_entry), file_part);
 }
+
+
+/**
+ * _gtk_file_chooser_entry_set_action:
+ * @chooser_entry: a #GtkFileChooserEntry
+ * @action: the action which is performed by the file selector using this entry
+ *
+ * Sets action which is performed by the file selector using this entry. 
+ * The #GtkFileChooserEntry will use different completion strategies for 
+ * different actions.
+ **/
+void
+_gtk_file_chooser_entry_set_action (GtkFileChooserEntry *chooser_entry,
+				    GtkFileChooserAction action)
+{
+  g_return_if_fail (GTK_IS_FILE_CHOOSER_ENTRY (chooser_entry));
+  
+  if (  chooser_entry->action != action)
+    {
+      chooser_entry->action = action;
+    }
+}
+
+
+/**
+ * _gtk_file_chooser_entry_get_action:
+ * @chooser_entry: a #GtkFileChooserEntry
+ *
+ * Gets the action for this entry. 
+ *
+ * Returns: the action
+ **/
+GtkFileChooserAction
+_gtk_file_chooser_entry_get_action (GtkFileChooserEntry *chooser_entry)
+{
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER_ENTRY (chooser_entry),
+			GTK_FILE_CHOOSER_ACTION_OPEN);
+  
+  return chooser_entry->action;
+}
