@@ -34,6 +34,7 @@
 #  include <stdio.h>
 #endif
 
+#include "gtkalias.h"
 #include "gtkprogressbar.h"
 #include "gtkintl.h"
 
@@ -53,6 +54,7 @@ enum {
   PROP_PULSE_STEP,
   PROP_ORIENTATION,
   PROP_TEXT,
+  PROP_ELLIPSIZE,
   
   /* Deprecated args */
   PROP_ADJUSTMENT,
@@ -78,6 +80,15 @@ static void gtk_progress_bar_real_update   (GtkProgress         *progress);
 static void gtk_progress_bar_paint         (GtkProgress         *progress);
 static void gtk_progress_bar_act_mode_enter (GtkProgress        *progress);
 
+static void gtk_progress_bar_set_bar_style_internal       (GtkProgressBar *pbar,
+							   GtkProgressBarStyle style);
+static void gtk_progress_bar_set_discrete_blocks_internal (GtkProgressBar *pbar,
+							   guint           blocks);
+static void gtk_progress_bar_set_activity_step_internal   (GtkProgressBar *pbar,
+							   guint           step);
+static void gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
+							   guint           blocks);
+
 
 GType
 gtk_progress_bar_get_type (void)
@@ -130,16 +141,16 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_ADJUSTMENT,
                                    g_param_spec_object ("adjustment",
-                                                        _("Adjustment"),
-                                                        _("The GtkAdjustment connected to the progress bar (Deprecated)"),
+                                                        P_("Adjustment"),
+                                                        P_("The GtkAdjustment connected to the progress bar (Deprecated)"),
                                                         GTK_TYPE_ADJUSTMENT,
                                                         G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ORIENTATION,
                                    g_param_spec_enum ("orientation",
-						      _("Orientation"),
-						      _("Orientation and growth direction of the progress bar"),
+						      P_("Orientation"),
+						      P_("Orientation and growth direction of the progress bar"),
 						      GTK_TYPE_PROGRESS_BAR_ORIENTATION,
 						      GTK_PROGRESS_LEFT_TO_RIGHT,
 						      G_PARAM_READWRITE));
@@ -147,8 +158,8 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_BAR_STYLE,
                                    g_param_spec_enum ("bar_style",
-						      _("Bar style"),
-						      _("Specifies the visual style of the bar in percentage mode (Deprecated)"),
+						      P_("Bar style"),
+						      P_("Specifies the visual style of the bar in percentage mode (Deprecated)"),
 						      GTK_TYPE_PROGRESS_BAR_STYLE,
 						      GTK_PROGRESS_CONTINUOUS,
 						      G_PARAM_READWRITE));
@@ -156,8 +167,8 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_STEP,
                                    g_param_spec_uint ("activity_step",
-						      _("Activity Step"),
-						      _("The increment used for each iteration in activity mode (Deprecated)"),
+						      P_("Activity Step"),
+						      P_("The increment used for each iteration in activity mode (Deprecated)"),
 						      -G_MAXUINT,
 						      G_MAXUINT,
 						      3,
@@ -166,8 +177,8 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_BLOCKS,
                                    g_param_spec_uint ("activity_blocks",
-						      _("Activity Blocks"),
-						      _("The number of blocks which can fit in the progress bar area in activity mode (Deprecated)"),
+						      P_("Activity Blocks"),
+						      P_("The number of blocks which can fit in the progress bar area in activity mode (Deprecated)"),
 						      2,
 						      G_MAXUINT,
 						      5,
@@ -176,8 +187,8 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_DISCRETE_BLOCKS,
                                    g_param_spec_uint ("discrete_blocks",
-						      _("Discrete Blocks"),
-						      _("The number of discrete blocks in a progress bar (when shown in the discrete style)"),
+						      P_("Discrete Blocks"),
+						      P_("The number of discrete blocks in a progress bar (when shown in the discrete style)"),
 						      2,
 						      G_MAXUINT,
 						      10,
@@ -186,8 +197,8 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
 				   PROP_FRACTION,
 				   g_param_spec_double ("fraction",
-							_("Fraction"),
-							_("The fraction of total work that has been completed"),
+							P_("Fraction"),
+							P_("The fraction of total work that has been completed"),
 							0.0,
 							1.0,
 							0.0,
@@ -196,8 +207,8 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
 				   PROP_PULSE_STEP,
 				   g_param_spec_double ("pulse_step",
-							_("Pulse Step"),
-							_("The fraction of total progress to move the bouncing block when pulsed"),
+							P_("Pulse Step"),
+							P_("The fraction of total progress to move the bouncing block when pulsed"),
 							0.0,
 							1.0,
 							0.1,
@@ -206,11 +217,34 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
 				   PROP_TEXT,
 				   g_param_spec_string ("text",
-							_("Text"),
-							_("Text to be displayed in the progress bar"),
+							P_("Text"),
+							P_("Text to be displayed in the progress bar"),
 							"%P %%",
 							G_PARAM_READWRITE));
 
+  /**
+   * GtkProgressBar:ellipsize:
+   *
+   * The preferred place to ellipsize the string, if the progressbar does 
+   * not have enough room to display the entire string, specified as a 
+   * #PangoEllisizeMode. 
+   *
+   * Note that setting this property to a value other than 
+   * %PANGO_ELLIPSIZE_NONE has the side-effect that the progressbar requests 
+   * only enough space to display the ellipsis "...". Another means to set a 
+   * progressbar's width is gtk_widget_set_size_request().
+   *
+   * Since: 2.6
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_ELLIPSIZE,
+                                   g_param_spec_enum ("ellipsize",
+                                                      P_("Ellipsize"),
+                                                      P_("The preferred place to ellipsize the string, if the progressbar does not have enough room to display the entire string, if at all"),
+						      PANGO_TYPE_ELLIPSIZE_MODE,
+						      PANGO_ELLIPSIZE_NONE,
+                                                      G_PARAM_READWRITE));
+
 }
 
 static void
@@ -225,6 +259,7 @@ gtk_progress_bar_init (GtkProgressBar *pbar)
   pbar->activity_dir = 1;
   pbar->activity_step = 3;
   pbar->activity_blocks = 5;
+  pbar->ellipsize = PANGO_ELLIPSIZE_NONE;
 }
 
 static void
@@ -247,16 +282,16 @@ gtk_progress_bar_set_property (GObject      *object,
       gtk_progress_bar_set_orientation (pbar, g_value_get_enum (value));
       break;
     case PROP_BAR_STYLE:
-      gtk_progress_bar_set_bar_style (pbar, g_value_get_enum (value));
+      gtk_progress_bar_set_bar_style_internal (pbar, g_value_get_enum (value));
       break;
     case PROP_ACTIVITY_STEP:
-      gtk_progress_bar_set_activity_step (pbar, g_value_get_uint (value));
+      gtk_progress_bar_set_activity_step_internal (pbar, g_value_get_uint (value));
       break;
     case PROP_ACTIVITY_BLOCKS:
-      gtk_progress_bar_set_activity_blocks (pbar, g_value_get_uint (value));
+      gtk_progress_bar_set_activity_blocks_internal (pbar, g_value_get_uint (value));
       break;
     case PROP_DISCRETE_BLOCKS:
-      gtk_progress_bar_set_discrete_blocks (pbar, g_value_get_uint (value));
+      gtk_progress_bar_set_discrete_blocks_internal (pbar, g_value_get_uint (value));
       break;
     case PROP_FRACTION:
       gtk_progress_bar_set_fraction (pbar, g_value_get_double (value));
@@ -267,6 +302,9 @@ gtk_progress_bar_set_property (GObject      *object,
     case PROP_TEXT:
       gtk_progress_bar_set_text (pbar, g_value_get_string (value));
       break;
+    case PROP_ELLIPSIZE:
+      gtk_progress_bar_set_ellipsize (pbar, g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -312,6 +350,9 @@ gtk_progress_bar_get_property (GObject      *object,
     case PROP_TEXT:
       g_value_set_string (value, gtk_progress_bar_get_text (pbar));
       break;
+    case PROP_ELLIPSIZE:
+      g_value_set_enum (value, pbar->ellipsize);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -452,6 +493,7 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   gchar *buf;
   PangoRectangle logical_rect;
   PangoLayout *layout;
+  gint width, height;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (widget));
   g_return_if_fail (requisition != NULL);
@@ -459,6 +501,9 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   progress = GTK_PROGRESS (widget);
   pbar = GTK_PROGRESS_BAR (widget);
 
+  width = 2 * widget->style->xthickness + 3 + 2 * TEXT_SPACING;
+  height = 2 * widget->style->ythickness + 3 + 2 * TEXT_SPACING;
+
   if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
     {
       if (!progress->adjustment)
@@ -467,8 +512,29 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
       buf = gtk_progress_get_text_from_value (progress, progress->adjustment->upper);
 
       layout = gtk_widget_create_pango_layout (widget, buf);
+
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	  
+      if (pbar->ellipsize)
+	{
+	  PangoContext *context;
+	  PangoFontMetrics *metrics;
+	  gint char_width;
+	  
+	  /* The minimum size for ellipsized text is ~ 3 chars */
+	  context = pango_layout_get_context (layout);
+	  metrics = pango_context_get_metrics (context, widget->style->font_desc, NULL);
+	  
+	  char_width = pango_font_metrics_get_approximate_char_width (metrics);
+	  pango_font_metrics_unref (metrics);
+	  
+	  width += PANGO_PIXELS (char_width) * 3;
+	}
+      else
+	width += logical_rect.width;
+      
+      height += logical_rect.height;
+
       g_object_unref (layout);
       g_free (buf);
     }
@@ -478,13 +544,8 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
     {
       if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
 	{
-	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH,
-				    2 * widget->style->xthickness + 3 +
-				    logical_rect.width + 2 * TEXT_SPACING);
-
-	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT,
-				     2 * widget->style->ythickness + 3 +
-				     logical_rect.height + 2 * TEXT_SPACING);
+	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
+	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
 	}
       else
 	{
@@ -496,13 +557,8 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
     {
       if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
 	{	  
-	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH,
-				    2 * widget->style->xthickness + 3 +
-				    logical_rect.width + 2 * TEXT_SPACING);
-
-	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT,
-				     2 * widget->style->ythickness + 3 +
-				     logical_rect.height + 2 * TEXT_SPACING);
+	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
+	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
 	}
       else
 	{
@@ -571,19 +627,242 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
     }
 }
 
+static void
+gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
+				 GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  GtkProgress *progress = GTK_PROGRESS (pbar);
+  gint x, y, w, h;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      x = pbar->activity_pos;
+      y = widget->style->ythickness;
+      w = MAX (2, widget->allocation.width / pbar->activity_blocks);
+      h = widget->allocation.height - 2 * widget->style->ythickness;
+      break;
+
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      x = widget->style->xthickness;
+      y = pbar->activity_pos;
+      w = widget->allocation.width - 2 * widget->style->xthickness;
+      h = MAX (2, widget->allocation.height / pbar->activity_blocks);
+      break;
+
+    default:
+      return;
+      break;
+    }
+
+  gtk_paint_box (widget->style,
+		 progress->offscreen_pixmap,
+		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
+		 NULL, widget, "bar",
+		 x, y, w, h);
+}
+
+static void
+gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
+				   gint                       amount,
+				   GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  gint x, y, w, h;
+
+  if (amount <= 0)
+    return;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      w = amount;
+      h = widget->allocation.height - widget->style->ythickness * 2;
+      y = widget->style->ythickness;
+      
+      x = widget->style->xthickness;
+      if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	x = widget->allocation.width - amount - x;
+      break;
+      
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      w = widget->allocation.width - widget->style->xthickness * 2;
+      h = amount;
+      x = widget->style->xthickness;
+      
+      y = widget->style->ythickness;
+      if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
+	y = widget->allocation.height - amount - y;
+      break;
+      
+    default:
+      return;
+      break;
+    }
+  
+  gtk_paint_box (widget->style,
+		 GTK_PROGRESS (pbar)->offscreen_pixmap,
+		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
+		 NULL, widget, "bar",
+		 x, y, w, h);
+}
+
+static void
+gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
+				 GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  gint i;
+
+  for (i = 0; i <= pbar->in_block; i++)
+    {
+      gint x, y, w, h, space;
+
+      switch (orientation)
+	{
+	case GTK_PROGRESS_LEFT_TO_RIGHT:
+	case GTK_PROGRESS_RIGHT_TO_LEFT:
+	  space = widget->allocation.width - 2 * widget->style->xthickness;
+	  
+	  x = widget->style->xthickness + (i * space) / pbar->blocks;
+	  y = widget->style->ythickness;
+	  w = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - x;
+	  h = widget->allocation.height - 2 * widget->style->ythickness;
+
+	  if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	    x = widget->allocation.width - w - x;
+	  break;
+	  
+	case GTK_PROGRESS_TOP_TO_BOTTOM:
+	case GTK_PROGRESS_BOTTOM_TO_TOP:
+	  space = widget->allocation.height - 2 * widget->style->ythickness;
+	  
+	  x = widget->style->xthickness;
+	  y = widget->style->ythickness + (i * space) / pbar->blocks;
+	  w = widget->allocation.width - 2 * widget->style->xthickness;
+	  h = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - y;
+	  
+	  if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
+	    y = widget->allocation.height - h - y;
+	  break;
+
+	default:
+	  return;
+	  break;
+	}
+      
+      gtk_paint_box (widget->style,
+		     GTK_PROGRESS (pbar)->offscreen_pixmap,
+		     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
+		     NULL, widget, "bar",
+		     x, y, w, h);
+    }
+}
+
+static void
+gtk_progress_bar_paint_text (GtkProgressBar            *pbar,
+			     gint			amount,
+			     GtkProgressBarOrientation  orientation)
+{
+  GtkProgress *progress = GTK_PROGRESS (pbar);
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  
+  gint x;
+  gint y;
+  gchar *buf;
+  GdkRectangle rect;
+  PangoLayout *layout;
+  PangoRectangle logical_rect;
+  GdkRectangle prelight_clip, normal_clip;
+  
+  buf = gtk_progress_get_current_text (progress);
+  
+  layout = gtk_widget_create_pango_layout (widget, buf);
+  pango_layout_set_ellipsize (layout, pbar->ellipsize);
+  if (pbar->ellipsize)
+    pango_layout_set_width (layout, widget->allocation.width * PANGO_SCALE);
+
+  pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+  
+  x = widget->style->xthickness + 1 +
+    (widget->allocation.width - 2 * widget->style->xthickness -
+     2 - logical_rect.width)
+    * progress->x_align; 
+
+  y = widget->style->ythickness + 1 +
+    (widget->allocation.height - 2 * widget->style->ythickness -
+     2 - logical_rect.height)
+    * progress->y_align;
+
+  rect.x = widget->style->xthickness;
+  rect.y = widget->style->ythickness;
+  rect.width = widget->allocation.width - 2 * widget->style->xthickness;
+  rect.height = widget->allocation.height - 2 * widget->style->ythickness;
+
+  prelight_clip = normal_clip = rect;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+      prelight_clip.width = amount;
+      normal_clip.x += amount;
+      normal_clip.width -= amount;
+      break;
+      
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      normal_clip.width -= amount;
+      prelight_clip.x += normal_clip.width;
+      prelight_clip.width -= normal_clip.width;
+      break;
+       
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+      prelight_clip.height = amount;
+      normal_clip.y += amount;
+      normal_clip.height -= amount;
+      break;
+      
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      normal_clip.height -= amount;
+      prelight_clip.y += normal_clip.height;
+      prelight_clip.height -= normal_clip.height;
+      break;
+    }
+  
+  gtk_paint_layout (widget->style,
+		    progress->offscreen_pixmap,
+		    GTK_STATE_PRELIGHT,
+		    FALSE,
+		    &prelight_clip,
+		    widget,
+		    "progressbar",
+		    x, y,
+		    layout);
+  
+  gtk_paint_layout (widget->style,
+		    progress->offscreen_pixmap,
+		    GTK_STATE_NORMAL,
+		    FALSE,
+		    &normal_clip,
+		    widget,
+		    "progressbar",
+		    x, y,
+		    layout);
+
+  g_object_unref (layout);
+  g_free (buf);
+}
+
 static void
 gtk_progress_bar_paint (GtkProgress *progress)
 {
   GtkProgressBar *pbar;
   GtkWidget *widget;
-  gint amount;
-  gint block_delta = 0;
-  gint space = 0;
-  gint i;
-  gint x;
-  gint y;
-  gdouble percentage;
-  gint size;
+
   GtkProgressBarOrientation orientation;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (progress));
@@ -600,16 +879,6 @@ gtk_progress_bar_paint (GtkProgress *progress)
 	orientation = GTK_PROGRESS_LEFT_TO_RIGHT;
     }
  
-  if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
-      orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-    space = widget->allocation.width -
-      2 * widget->style->xthickness;
-  else
-    space = widget->allocation.height -
-      2 * widget->style->ythickness;
-
-  percentage = gtk_progress_get_current_percentage (progress);
-
   if (progress->offscreen_pixmap)
     {
       gtk_paint_box (widget->style,
@@ -622,248 +891,94 @@ gtk_progress_bar_paint (GtkProgress *progress)
       
       if (progress->activity_mode)
 	{
+	  gtk_progress_bar_paint_activity (pbar, orientation);
+	}
+      else
+	{
+	  gint amount;
+	  gint space;
+	  
 	  if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
 	      orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	    space = widget->allocation.width - 2 * widget->style->xthickness;
+	  else
+	    space = widget->allocation.height - 2 * widget->style->ythickness;
+	  
+	  amount = space *
+	    gtk_progress_get_current_percentage (GTK_PROGRESS (pbar));
+	  
+	  if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
 	    {
-	      size = MAX (2, widget->allocation.width / pbar->activity_blocks);
-	      
-	      gtk_paint_box (widget->style,
-			     progress->offscreen_pixmap,
-			     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-			     NULL, widget, "bar",
-			     pbar->activity_pos,
-			     widget->style->ythickness,
-			     size,
-			     widget->allocation.height - widget->style->ythickness * 2);
-	      return;
+	      gtk_progress_bar_paint_continuous (pbar, amount, orientation);
+
+	      if (GTK_PROGRESS (pbar)->show_text)
+		gtk_progress_bar_paint_text (pbar, amount, orientation);
 	    }
 	  else
-	    {
-	      size = MAX (2, widget->allocation.height / pbar->activity_blocks);
-	      
-	      gtk_paint_box (widget->style,
-			     progress->offscreen_pixmap,
-			     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-			     NULL, widget, "bar",
-			     widget->style->xthickness,
-			     pbar->activity_pos,
-			     widget->allocation.width - widget->style->xthickness * 2,
-			     size);
-	      return;
-	    }
+	    gtk_progress_bar_paint_discrete (pbar, orientation);
 	}
-      
-      amount = percentage * space;
-      
-      if (amount > 0)
-	{
-	  switch (orientation)
-	    {
-	      
-	    case GTK_PROGRESS_LEFT_TO_RIGHT:
-	      
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->style->xthickness,
-				 widget->style->ythickness,
-				 amount,
-				 widget->allocation.height - widget->style->ythickness * 2);
-		}
-	      else
-		{
-		  x = widget->style->xthickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      block_delta = (((i + 1) * space) / pbar->blocks)
-			- ((i * space) / pbar->blocks);
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     x,
-				     widget->style->ythickness,
-				     block_delta,
-				     widget->allocation.height - widget->style->ythickness * 2);
-		      
-		      x +=  block_delta;
-		    }
-		}
-	      break;
-	      
-	    case GTK_PROGRESS_RIGHT_TO_LEFT:
-	      
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->allocation.width - 
-				 widget->style->xthickness - amount,
-				 widget->style->ythickness,
-				 amount,
-				 widget->allocation.height -
-				 widget->style->ythickness * 2);
-		}
-	      else
-		{
-		  x = widget->allocation.width - 
-		    widget->style->xthickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      block_delta = (((i + 1) * space) / pbar->blocks) -
-			((i * space) / pbar->blocks);
-		      
-		      x -=  block_delta;
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     x,
-				     widget->style->ythickness,
-				     block_delta,
-				     widget->allocation.height -
-				     widget->style->ythickness * 2);
-		    }
-		}
-	      break;
+    }
+}
 
-	    case GTK_PROGRESS_BOTTOM_TO_TOP:
+static void
+gtk_progress_bar_set_bar_style_internal (GtkProgressBar     *pbar,
+					 GtkProgressBarStyle bar_style)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
 
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->style->xthickness,
-				 widget->allocation.height - 
-				 widget->style->ythickness - amount,
-				 widget->allocation.width -
-				 widget->style->xthickness * 2,
-				 amount);
-		}
-	      else
-		{
-		  y = widget->allocation.height - 
-		    widget->style->ythickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      block_delta = (((i + 1) * space) / pbar->blocks) -
-			((i * space) / pbar->blocks);
-		      
-		      y -= block_delta;
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     widget->style->xthickness,
-				     y,
-				     widget->allocation.width - 
-				     widget->style->xthickness * 2,
-				     block_delta);
-		    }
-		}
-	      break;
-	      
-	    case GTK_PROGRESS_TOP_TO_BOTTOM:
-	      
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->style->xthickness,
-				 widget->style->ythickness,
-				 widget->allocation.width -
-				 widget->style->xthickness * 2,
-				 amount);
-		}
-	      else
-		{
-		  y = widget->style->ythickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      
-		      block_delta = (((i + 1) * space) / pbar->blocks)
-			- ((i * space) / pbar->blocks);
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     widget->style->xthickness,
-				     y,
-				     widget->allocation.width -
-				     widget->style->xthickness * 2,
-				     block_delta);
-		      
-		      y += block_delta;
-		    }
-		}
-	      break;
-	      
-	    default:
-	      break;
-	    }
-	}
-      
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{
-	  gint x;
-	  gint y;
-	  gchar *buf;
-	  GdkRectangle rect;
-	  PangoLayout *layout;
-	  PangoRectangle logical_rect;
+  if (pbar->bar_style != bar_style)
+    {
+      pbar->bar_style = bar_style;
 
-	  buf = gtk_progress_get_current_text (progress);
+      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
+	gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-	  layout = gtk_widget_create_pango_layout (widget, buf);
-	  pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
-	  
-	  x = widget->style->xthickness + 1 + 
-	    (widget->allocation.width - 2 * widget->style->xthickness -
-	     3 - logical_rect.width)
-	    * progress->x_align; 
+      g_object_notify (G_OBJECT (pbar), "bar_style");
+    }
+}
 
-	  y = widget->style->ythickness + 1 +
-	    (widget->allocation.height - 2 * widget->style->ythickness -
-	     3 - logical_rect.height)
-	    * progress->y_align;
+static void
+gtk_progress_bar_set_discrete_blocks_internal (GtkProgressBar *pbar,
+					       guint           blocks)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (blocks > 1);
 
-	  rect.x = widget->style->xthickness + 1;
-	  rect.y = widget->style->ythickness + 1;
-	  rect.width = widget->allocation.width -
-	    2 * widget->style->xthickness - 3;
-	  rect.height = widget->allocation.height -
-	    2 * widget->style->ythickness - 3;
-      
-          gtk_paint_layout (widget->style,
-                            progress->offscreen_pixmap,
-                            GTK_WIDGET_STATE (widget),
-			    FALSE,
-                            &rect,
-                            widget,
-                            "progressbar",
-                            x, y,
-                            layout);
+  if (pbar->blocks != blocks)
+    {
+      pbar->blocks = blocks;
 
-          g_object_unref (layout);
-	  g_free (buf);
- 	}
+      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
+	gtk_widget_queue_resize (GTK_WIDGET (pbar));
+
+      g_object_notify (G_OBJECT (pbar), "discrete_blocks");
+    }
+}
+
+static void
+gtk_progress_bar_set_activity_step_internal (GtkProgressBar *pbar,
+					     guint           step)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+
+  if (pbar->activity_step != step)
+    {
+      pbar->activity_step = step;
+      g_object_notify (G_OBJECT (pbar), "activity_step");
+    }
+}
+
+static void
+gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
+					       guint           blocks)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (blocks > 1);
+
+  if (pbar->activity_blocks != blocks)
+    {
+      pbar->activity_blocks = blocks;
+      g_object_notify (G_OBJECT (pbar), "activity_blocks");
     }
 }
 
@@ -1078,15 +1193,7 @@ gtk_progress_bar_set_bar_style (GtkProgressBar     *pbar,
 {
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
 
-  if (pbar->bar_style != bar_style)
-    {
-      pbar->bar_style = bar_style;
-
-      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
-	gtk_widget_queue_resize (GTK_WIDGET (pbar));
-
-      g_object_notify (G_OBJECT (pbar), "bar_style");
-    }
+  gtk_progress_bar_set_bar_style_internal (pbar, bar_style);
 }
 
 void
@@ -1096,15 +1203,7 @@ gtk_progress_bar_set_discrete_blocks (GtkProgressBar *pbar,
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
   g_return_if_fail (blocks > 1);
 
-  if (pbar->blocks != blocks)
-    {
-      pbar->blocks = blocks;
-
-      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
-	gtk_widget_queue_resize (GTK_WIDGET (pbar));
-
-      g_object_notify (G_OBJECT (pbar), "discrete_blocks");
-    }
+  gtk_progress_bar_set_discrete_blocks_internal (pbar, blocks);
 }
 
 void
@@ -1113,11 +1212,7 @@ gtk_progress_bar_set_activity_step (GtkProgressBar *pbar,
 {
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
 
-  if (pbar->activity_step != step)
-    {
-      pbar->activity_step = step;
-      g_object_notify (G_OBJECT (pbar), "activity_step");
-    }
+  gtk_progress_bar_set_activity_step_internal (pbar, step);
 }
 
 void
@@ -1127,9 +1222,52 @@ gtk_progress_bar_set_activity_blocks (GtkProgressBar *pbar,
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
   g_return_if_fail (blocks > 1);
 
-  if (pbar->activity_blocks != blocks)
+  gtk_progress_bar_set_activity_blocks_internal (pbar, blocks);
+}
+
+/**
+ * gtk_progress_bar_set_ellipsize:
+ * @pbar: a #GtkProgressBar
+ * @mode: a #PangoEllipsizeMode
+ *
+ * Sets the mode used to ellipsize (add an ellipsis: "...") the text 
+ * if there is not enough space to render the entire string.
+ *
+ * Since: 2.6
+ **/
+void
+gtk_progress_bar_set_ellipsize (GtkProgressBar     *pbar,
+				PangoEllipsizeMode  mode)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (mode >= PANGO_ELLIPSIZE_NONE && 
+		    mode <= PANGO_ELLIPSIZE_END);
+  
+  if ((PangoEllipsizeMode)pbar->ellipsize != mode)
     {
-      pbar->activity_blocks = blocks;
-      g_object_notify (G_OBJECT (pbar), "activity_blocks");
+      pbar->ellipsize = mode;
+
+      g_object_notify (G_OBJECT (pbar), "ellipsize");
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
     }
 }
+
+/**
+ * gtk_pgrogress_bar_get_ellipsize:
+ * @pbar: a #GtkProgressBar
+ *
+ * Returns the ellipsizing position of the progressbar. 
+ * See gtk_progress_bar_set_ellipsize().
+ *
+ * Return value: #PangoEllipsizeMode
+ *
+ * Since: 2.6
+ **/
+PangoEllipsizeMode 
+gtk_progress_bar_get_ellipsize (GtkProgressBar *pbar)
+{
+  g_return_val_if_fail (GTK_IS_PROGRESS_BAR (pbar), PANGO_ELLIPSIZE_NONE);
+
+  return pbar->ellipsize;
+}
+
