@@ -22,6 +22,7 @@
 #undef GTK_DISABLE_DEPRECATED
 
 #include <config.h>
+#include "gtkalias.h"
 #include "gtkseparatormenuitem.h"
 #include "gtkseparatortoolitem.h"
 #include "gtkintl.h"
@@ -229,12 +230,17 @@ gtk_separator_tool_item_expose (GtkWidget      *widget,
 				GdkEventExpose *event)
 {
   GtkToolbar *toolbar = NULL;
+  GtkSeparatorToolItemPrivate *priv =
+      GTK_SEPARATOR_TOOL_ITEM_GET_PRIVATE (widget);
 
-  if (widget->parent && GTK_IS_TOOLBAR (widget->parent))
-    toolbar = GTK_TOOLBAR (widget->parent);
+  if (priv->draw)
+    {
+      if (widget->parent && GTK_IS_TOOLBAR (widget->parent))
+	toolbar = GTK_TOOLBAR (widget->parent);
 
-  _gtk_toolbar_paint_space_line (widget, toolbar,
-				 &(event->area), &widget->allocation);
+      _gtk_toolbar_paint_space_line (widget, toolbar,
+				     &(event->area), &widget->allocation);
+    }
   
   return FALSE;
 }
