@@ -29,6 +29,7 @@
 #include <string.h>
 
 #include "gdk/gdkkeysyms.h"
+#include "gtkalias.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkwindow.h"
@@ -106,7 +107,7 @@ static guint socket_signals[LAST_SIGNAL] = { 0 };
 
 static GtkWidgetClass *parent_class = NULL;
 
-GtkSocketPrivate *
+static GtkSocketPrivate *
 gtk_socket_get_private (GtkSocket *socket)
 {
   GtkSocketPrivate *private;
@@ -401,6 +402,8 @@ gtk_socket_end_embedding (GtkSocket *socket)
 
   g_object_unref (socket->plug_window);
   socket->plug_window = NULL;
+  socket->current_width = 0;
+  socket->current_height = 0;
   private->resize_count = 0;
 
   /* Remove from end to avoid indexes shifting. This is evil */
