@@ -1,7 +1,9 @@
 
+#include <config.h>
+#include "gtkalias.h"
 #include "gtktexttagtable.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
+#include "gtktextbuffer.h" /* just for the lame notify_will_remove_tag hack */
 
 #include <stdlib.h>
 
@@ -51,10 +53,8 @@ gtk_text_tag_table_get_type (void)
         (GInstanceInitFunc) gtk_text_tag_table_init
       };
 
-      our_type = g_type_register_static (G_TYPE_OBJECT,
-                                         "GtkTextTagTable",
-                                         &our_info,
-                                         0);
+      our_type = g_type_register_static (G_TYPE_OBJECT, "GtkTextTagTable",
+                                         &our_info, 0);
     }
 
   return our_type;
@@ -86,23 +86,23 @@ gtk_text_tag_table_class_init (GtkTextTagTableClass *klass)
 
   signals[TAG_ADDED] =
     g_signal_new ("tag_added",
-                  GTK_CLASS_TYPE (object_class),
+                  G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkTextTagTableClass, tag_added),
                   NULL, NULL,
                   _gtk_marshal_VOID__OBJECT,
-                  GTK_TYPE_NONE,
+                  G_TYPE_NONE,
                   1,
                   GTK_TYPE_TEXT_TAG);
 
   signals[TAG_REMOVED] =
     g_signal_new ("tag_removed",                   
-                  GTK_CLASS_TYPE (object_class),
+                  G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkTextTagTableClass, tag_removed),
                   NULL, NULL,
                   _gtk_marshal_VOID__OBJECT,
-                  GTK_TYPE_NONE,
+                  G_TYPE_NONE,
                   1,
                   GTK_TYPE_TEXT_TAG);
 }
@@ -126,7 +126,7 @@ gtk_text_tag_table_new (void)
 {
   GtkTextTagTable *table;
 
-  table = GTK_TEXT_TAG_TABLE (g_object_new (gtk_text_tag_table_get_type (), NULL));
+  table = g_object_new (GTK_TYPE_TEXT_TAG_TABLE, NULL);
 
   return table;
 }
@@ -134,12 +134,23 @@ gtk_text_tag_table_new (void)
 static void
 foreach_unref (GtkTextTag *tag, gpointer data)
 {
+  GSList *tmp;
+  
   /* We don't want to emit the remove signal here; so we just unparent
    * and unref the tag.
    */
 
+  tmp = tag->table->buffers;
+  while (tmp != NULL)
+    {
+      _gtk_text_buffer_notify_will_remove_tag (GTK_TEXT_BUFFER (tmp->data),
+                                               tag);
+      
+      tmp = tmp->next;
+    }
+  
   tag->table = NULL;
-  g_object_unref (G_OBJECT (tag));
+  g_object_unref (tag);
 }
 
 static void
@@ -148,12 +159,14 @@ gtk_text_tag_table_finalize (GObject *object)
   GtkTextTagTable *table;
 
   table = GTK_TEXT_TAG_TABLE (object);
-
+  
   gtk_text_tag_table_foreach (table, foreach_unref, NULL);
 
   g_hash_table_destroy (table->hash);
   g_slist_free (table->anonymous);
 
+  g_slist_free (table->buffers);
+  
   (* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 static void
@@ -223,7 +236,7 @@ gtk_text_tag_table_add (GtkTextTagTable *table,
       return;
     }
   
-  g_object_ref (G_OBJECT (tag));
+  g_object_ref (tag);
 
   if (tag->name)
     g_hash_table_insert (table->hash, tag->name, tag);
@@ -242,7 +255,7 @@ gtk_text_tag_table_add (GtkTextTagTable *table,
   g_assert (size > 0);
   tag->priority = size - 1;
 
-  g_signal_emit (G_OBJECT (table), signals[TAG_ADDED], 0, tag);
+  g_signal_emit (table, signals[TAG_ADDED], 0, tag);
 }
 
 /**
@@ -277,10 +290,24 @@ void
 gtk_text_tag_table_remove (GtkTextTagTable *table,
                            GtkTextTag      *tag)
 {
+  GSList *tmp;
+  
   g_return_if_fail (GTK_IS_TEXT_TAG_TABLE (table));
   g_return_if_fail (GTK_IS_TEXT_TAG (tag));
   g_return_if_fail (tag->table == table);
 
+  /* Our little bad hack to be sure buffers don't still have the tag
+   * applied to text in the buffer
+   */
+  tmp = table->buffers;
+  while (tmp != NULL)
+    {
+      _gtk_text_buffer_notify_will_remove_tag (GTK_TEXT_BUFFER (tmp->data),
+                                               tag);
+      
+      tmp = tmp->next;
+    }
+  
   /* Set ourselves to the highest priority; this means
      when we're removed, there won't be any gaps in the
      priorities of the tags in the table. */
@@ -296,9 +323,9 @@ gtk_text_tag_table_remove (GtkTextTagTable *table,
       table->anon_count -= 1;
     }
 
-  g_signal_emit (G_OBJECT (table), signals[TAG_REMOVED], 0, tag);
+  g_signal_emit (table, signals[TAG_REMOVED], 0, tag);
 
-  g_object_unref (G_OBJECT (tag));
+  g_object_unref (tag);
 }
 
 struct ForeachData
@@ -368,3 +395,33 @@ gtk_text_tag_table_get_size (GtkTextTagTable *table)
 
   return g_hash_table_size (table->hash) + table->anon_count;
 }
+
+void
+_gtk_text_tag_table_add_buffer (GtkTextTagTable *table,
+                                gpointer         buffer)
+{
+  g_return_if_fail (GTK_IS_TEXT_TAG_TABLE (table));
+
+  table->buffers = g_slist_prepend (table->buffers, buffer);
+}
+
+static void
+foreach_remove_tag (GtkTextTag *tag, gpointer data)
+{
+  GtkTextBuffer *buffer;
+
+  buffer = GTK_TEXT_BUFFER (data);
+
+  _gtk_text_buffer_notify_will_remove_tag (buffer, tag);
+}
+
+void
+_gtk_text_tag_table_remove_buffer (GtkTextTagTable *table,
+                                   gpointer         buffer)
+{
+  g_return_if_fail (GTK_IS_TEXT_TAG_TABLE (table));
+
+  gtk_text_tag_table_foreach (table, foreach_remove_tag, buffer);
+  
+  table->buffers = g_slist_remove (table->buffers, buffer);
+}
