@@ -24,10 +24,12 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 
+#include "gtkalias.h"
 #include "gtklabel.h"
 #include "gtkmain.h"
 #include "gtkmenuitem.h"
@@ -379,10 +381,13 @@ gtk_tooltips_set_tip (GtkTooltips *tooltips,
 static gint
 gtk_tooltips_paint_window (GtkTooltips *tooltips)
 {
+  GtkRequisition req;
+
+  gtk_widget_size_request (tooltips->tip_window, &req);
   gtk_paint_flat_box (tooltips->tip_window->style, tooltips->tip_window->window,
 		      GTK_STATE_NORMAL, GTK_SHADOW_OUT, 
 		      NULL, GTK_WIDGET(tooltips->tip_window), "tooltip",
-		      0, 0, -1, -1);
+		      0, 0, req.width, req.height);
 
   return FALSE;
 }
