@@ -458,16 +458,13 @@ setup_menu_settings (GtkSettings * settings)
     if (SystemParametersInfo (SPI_GETMENUSHOWDELAY, 0, &menu_delay, 0)) {
       if (klazz) {
 	if (g_object_class_find_property (klazz, "gtk-menu-bar-popup-delay")) {
-	  g_object_set (G_OBJECT (settings), "gtk-menu-bar-popup-delay",
-			0, NULL);
+	  g_object_set (settings, "gtk-menu-bar-popup-delay", 0, NULL);
 	}
 	if (g_object_class_find_property (klazz, "gtk-menu-popup-delay")) {
-	  g_object_set (G_OBJECT (settings), "gtk-menu-popup-delay",
-			menu_delay, NULL);
+	  g_object_set (settings, "gtk-menu-popup-delay", menu_delay, NULL);
 	}
 	if (g_object_class_find_property (klazz, "gtk-menu-popdown-delay")) {
-	  g_object_set (G_OBJECT (settings), "gtk-menu-popdown-delay",
-			menu_delay, NULL);
+	  g_object_set (settings, "gtk-menu-popdown-delay", menu_delay, NULL);
 	}
       }
     }
@@ -485,19 +482,17 @@ msw_style_setup_system_settings (void)
     return;
 
   cursor_blink_time = GetCaretBlinkTime ();
-  g_object_set (G_OBJECT (settings), "gtk-cursor-blink",
-		        cursor_blink_time > 0, NULL);
+  g_object_set (settings, "gtk-cursor-blink", cursor_blink_time > 0, NULL);
 
   if (cursor_blink_time > 0)
   {
-  	g_object_set (G_OBJECT (settings), "gtk-cursor-blink-time",
-					2*cursor_blink_time, NULL);
+  	g_object_set (settings, "gtk-cursor-blink-time", 2*cursor_blink_time,
+		      NULL);
   }
 
-  g_object_set (G_OBJECT (settings), "gtk-double-click-time",
-		GetDoubleClickTime(), NULL);
-  g_object_set (G_OBJECT (settings), "gtk-dnd-drag-threshold",
-		GetSystemMetrics(SM_CXDRAG), NULL);
+  g_object_set (settings, "gtk-double-click-time", GetDoubleClickTime(), NULL);
+  g_object_set (settings, "gtk-dnd-drag-threshold", GetSystemMetrics(SM_CXDRAG),
+		NULL);
 
   setup_menu_settings (settings);
 
@@ -1169,6 +1164,29 @@ reverse_engineer_stepper_box (GtkWidget    *range,
   *height = box_height;
 }
 
+static XpThemeElement to_xp_arrow(GtkArrowType   arrow_type)
+{
+	XpThemeElement xp_arrow;
+
+	switch (arrow_type)
+	{
+		case GTK_ARROW_UP:
+			xp_arrow = XP_THEME_ELEMENT_ARROW_UP;
+			break;
+		case GTK_ARROW_DOWN:
+			xp_arrow = XP_THEME_ELEMENT_ARROW_DOWN;
+			break;
+		case GTK_ARROW_LEFT:
+			xp_arrow = XP_THEME_ELEMENT_ARROW_LEFT;
+			break;
+		default:
+			xp_arrow = XP_THEME_ELEMENT_ARROW_RIGHT;
+			break;
+	}
+
+	return xp_arrow;
+}
+
 static void
 draw_arrow (GtkStyle      *style,
 	    GdkWindow     *window,
@@ -1217,29 +1235,14 @@ draw_arrow (GtkStyle      *style,
       gint box_y = y;
       gint box_width = width;
       gint box_height = height;
-      XpThemeElement xp_arrow;
+
       reverse_engineer_stepper_box (widget, arrow_type,
 				    &box_x, &box_y, &box_width, &box_height);
 
       if (scrollbar->range.adjustment->page_size >= (scrollbar->range.adjustment->upper-scrollbar->range.adjustment->lower))
         is_disabled = TRUE;
 
-      switch (arrow_type)
-        {
-        case GTK_ARROW_UP:
-          xp_arrow = XP_THEME_ELEMENT_ARROW_UP;
-          break;
-        case GTK_ARROW_DOWN:
-          xp_arrow = XP_THEME_ELEMENT_ARROW_DOWN;
-          break;
-        case GTK_ARROW_LEFT:
-          xp_arrow = XP_THEME_ELEMENT_ARROW_LEFT;
-          break;
-        default:
-          xp_arrow = XP_THEME_ELEMENT_ARROW_RIGHT;
-          break;
-        }
-      if (xp_theme_draw(window, xp_arrow, style, box_x, box_y, box_width, box_height, state, area))
+      if (xp_theme_draw(window, to_xp_arrow(arrow_type), style, box_x, box_y, box_width, box_height, state, area))
         {
         }
       else if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN)
@@ -1267,6 +1270,13 @@ draw_arrow (GtkStyle      *style,
 		  if (xp_theme_draw(window, XP_THEME_ELEMENT_REBAR_CHEVRON, style, x, y, width, height, state, area))
 				return;
 	  }
+	  /* probably a gtk combo box on a toolbar */
+	  else if (widget->parent && GTK_IS_BUTTON (widget->parent))
+  		{
+		  	if (xp_theme_draw(window, XP_THEME_ELEMENT_COMBOBUTTON, style, x-3, widget->allocation.y+1,
+                        width+5, widget->allocation.height-4, state, area))
+				return;
+  		}
 
       if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN)
 	{
@@ -1789,7 +1799,7 @@ draw_resize_grip (GtkStyle      *style,
                        gint           height)
 {
 	if (detail && !strcmp(detail, "statusbar")) {
-		if (!xp_theme_draw(window, XP_THEME_ELEMENT_STATUS_GRIPPER, style, x, y, width, height,
+		if (xp_theme_draw(window, XP_THEME_ELEMENT_STATUS_GRIPPER, style, x, y, width, height,
                            state_type, area))
 			return;
 	}
@@ -1812,7 +1822,7 @@ draw_handle (GtkStyle        *style,
 	     gint             height,
 	     GtkOrientation   orientation)
 {
-  if (GTK_IS_HANDLE_BOX (widget))
+  if (is_toolbar_child (widget))
     {
       XpThemeElement hndl;
 
