@@ -217,7 +217,7 @@ animation_timer (GtkEntryCompletion *completion)
 	{
 	  g_print ("readding model!\n");
 	  
-	  gtk_entry_completion_set_model (completion, old_store);
+	  gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (old_store));
 	  g_object_unref (old_store);
 	  old_store = NULL;
 	}
@@ -237,8 +237,8 @@ animation_timer (GtkEntryCompletion *completion)
 	}
       else
 	{
-	  gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
-	  gtk_list_store_remove (store, &iter);
+	  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter))
+	    gtk_list_store_remove (store, &iter);
 	}
     }
   
@@ -337,12 +337,12 @@ main (int argc, char *argv[])
 				  "text", 1, NULL); 
   
   gtk_entry_completion_set_match_func (completion, match_func, NULL, NULL);
-  g_signal_connect (G_OBJECT (completion), "match-selected", 
+  g_signal_connect (completion, "match-selected", 
 		    G_CALLBACK (match_selected_cb), NULL);
 
   gtk_entry_completion_insert_action_text (completion, 100, "action!");
   gtk_entry_completion_insert_action_text (completion, 101, "'nother action!");
-  g_signal_connect (completion, "action_activated", G_CALLBACK (activated_cb), 0);
+  g_signal_connect (completion, "action_activated", G_CALLBACK (activated_cb), NULL);
 
   /* Create our third entry */
   entry = gtk_entry_new ();
